package restapi.controllers;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import restapi.json.*;
import restapi.services.ContactPreferencesService;
import restapi.services.ContactService;
import restapi.services.UserService;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.LoggedUser;
import restapi.utils.swagger.SwaggerContact;
import restapi.utils.swagger.SwaggerContacts;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@RequestMapping("/contact")
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
public class ContactPreferencesController {
    @NonNull
    private final UserService userService;
    @NonNull
    private final ContactPreferencesService contactPreferencesService;
    @NonNull
    private final ContactService contactService;
/*
    @ApiResponse(description = "", responseCode = "", headers = {}, links = {},
            content = {@Content(mediaType = "", examples = {@ExampleObject(name = "", summary = "", value = "", externalValue = "", extensions = {}, ref = "", description = "")}),
                    @Content(encoding = {}, extensions = {@Extension(name = "", properties = {@ExtensionProperty(name = "", value = "", parseValue = true)})})},
            extensions = {}, ref = "")

 */

    /**
     * Endpoint to get all contacts for User
     * endpoint /contact/all
     *
     * @param user Authenticated LoggedUser from token
     * @return user's contacts
     */
    @Operation(
            summary = "Get user's preferences for all their contacts",
            description = "Returns all contacts the user has in the app.",
            tags = {"Contacts"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER)
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "List of user's contacts with user's preferences",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerContacts.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/all")
    public ResponseEntity<RestResponse> getContacts(@AuthenticationPrincipal final LoggedUser user) {
        try {
            long id = userService.getIdByUsername(user.getUsername());
            RestList contacts = contactPreferencesService.getContacts(id);
            if (contacts.isEmpty()) {
                contactService.findUsersContacts(id).forEach((contactId) ->
                        contactPreferencesService.addPreferences(id, contactId));
                contacts = contactPreferencesService.getContacts(id);
            }
            RestDictionary dic = new RestDictionary();
            dic.put("contacts", contacts);
            return new ResponseEntity<>(dic, HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }

    /**
     * Endpoint to get the details of specific contact
     * endpoint /contact/details
     *
     * @param user      Authenticated LoggedUser from token
     * @param contactId id of contact to get the details from
     * @return contacts' details in form of ContactDTO
     * @see ContactDTO
     */
    @Operation(
            summary = "Get one contact's details & preferences",
            description = "Returns the preferences of the user for this contact and its user details, if this is contact of user.",
            tags = {"Contacts"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "contactId", description = "Id of contact we want to retrieve the preferences of.", required = true, schema = @Schema(implementation = Long.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Details of contact",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerContact.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/details")
    public ResponseEntity<RestResponse> getContactPreferences(@AuthenticationPrincipal final LoggedUser user,
                                                              @RequestParam final long contactId) {
        try {
            long id = userService.getIdByUsername(user.getUsername());
            ContactDTO contact = contactPreferencesService.getContactDTO(id, contactId);
            return new ResponseEntity<>(contact, HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }

    /**
     * Endpoint to update range for specific contact
     * endpoint /contact/range
     *
     * @param user      Authenticated LoggedUser from token
     * @param contactId Id of contact we want to change the range for
     * @param range     New range to save on database
     * @return Rest Message OK or Error 404
     */
    @Operation(
            summary = "Change range preference for contact",
            description = "Changes the range preference for a given contact, and updates the shared preferences.",
            tags = {"Contacts"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "contactId", description = "Id of contact we want to change the range for.", required = true, schema = @Schema(implementation = Long.class)),
                    @Parameter(name = "range", description = "New range to save on database.", required = true, schema = @Schema(implementation = Long.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Range preference has been successfully updated",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/range")
    public ResponseEntity<RestResponse> updateContactRange(@AuthenticationPrincipal final LoggedUser user,
                                                           @RequestParam final long contactId,
                                                           @RequestParam final long range) {
        try {
            long id = userService.getIdByUsername(user.getUsername());
            contactPreferencesService.changeParam(id, contactId, range);
            return new ResponseEntity<>(new RestMessage((short) 200, "updated"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }

    /**
     * Changes if a given contact is blocked by user or not, and updates the shared preferences. The end result of blocked is given by the parameter blocked.
     * endpoint /contact/block
     *
     * @param user      Authenticated LoggedUser from token
     * @param contactId Id of contact we want to block or unblock
     * @param blocked   True if the user wants to block contact, false otherwise
     * @return Rest Message OK or Error 404
     */
    @Operation(
            summary = "Change blocked parameter for contact",
            description = "Changes if a given contact is blocked by user or not, and updates the shared preferences. The end result of blocked is given by the parameter blocked.",
            tags = {"Contacts"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "contactId", description = "Id of contact we want to block or unblock.", required = true, schema = @Schema(implementation = Long.class)),
                    @Parameter(name = "blocked", description = "True if the user wants to block contact, false otherwise.", required = true, schema = @Schema(implementation = Boolean.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The contact has been successfully blocked or unblocked",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/block")
    public ResponseEntity<RestResponse> updateContactBlocked(@AuthenticationPrincipal final LoggedUser user,
                                                             @RequestParam final long contactId,
                                                             @RequestParam final boolean blocked) {
        try {
            long id = userService.getIdByUsername(user.getUsername());
            contactPreferencesService.changeBlocked(id, contactId, blocked);
            return new ResponseEntity<>(new RestMessage((short) 200, blocked ? "blocked" : "unblocked"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }
}
