package restapi.controllers;


import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import restapi.crud.UserEntity;
import restapi.json.*;
import restapi.services.ContactService;
import restapi.services.RequestService;
import restapi.services.UserService;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.LoggedUser;
import restapi.utils.services.phone.PhoneNumberService;
import restapi.utils.swagger.SwaggerContact;
import restapi.utils.swagger.SwaggerUsers;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@RequestMapping("/contacts")
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
public class ContactsController {
    @NonNull
    private final UserService userService;
    @NonNull
    private final ContactService contactService;
    @NonNull
    private final RequestService requestService;
    @NonNull
    private final PhoneNumberService phoneNumberService;
    private final Logger logger = Logger.getLogger(ContactsController.class.getName());

    /**
     * Returns all contacts the user has in the app.
     * endpoint /contacts/mine
     *
     * @param user Authenticated LoggedUser from token
     * @return Rest Message OK or Error 404
     */
    @Operation(
            summary = "Get all contacts",
            description = "Returns all contacts the user has in the app.",
            tags = {"Contacts"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER)
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "List of user's contacts",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerUsers.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/mine")
    public ResponseEntity<RestResponse> getContacts(@AuthenticationPrincipal final LoggedUser user) {
        try {
            long userId = userService.getIdByUsername(user.getUsername());
            List<Long> list = contactService.findUsersContacts(userId);
            RestList users = new RestList();
            for (long id : list) {
                users.add(userService.getDtoFromUserId(id));
            }
            RestDictionary ret = new RestDictionary();
            ret.put("contacts", users);
            return new ResponseEntity<>(ret, HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }

    /**
     * Get all users that have a phone matching in phones. Returns details for all user that have a phone that is in the list of phones passed by parameter.
     * endpoint /contacts/exist
     *
     * @param user   Authenticated LoggedUser from token
     * @param phones List of phones to search on the database
     * @return Details of contact
     */
    @Operation(
            summary = "Get all users that have a phone matching in phones",
            description = "Returns details for all user that have a phone that is in the list of phones passed by parameter.",
            tags = {"Contacts"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "phones", description = "List of phones to search on the database.", required = true, schema = @Schema(implementation = String[].class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Details of contact",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerContact.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/exist")
    public ResponseEntity<RestResponse> areContactsUsers(@AuthenticationPrincipal final LoggedUser user,
                                                         @RequestParam final List<String> phones) {
        try {
            UserEntity userEntity = userService.getUser(user.getUsername());
            RestList ret = new RestList();
            ArrayList<String> addedPhones = new ArrayList<>();
            for (String phone : phones) {
                if (!phoneNumberService.equalPhones(phone, userEntity.getPhone(), userEntity.getCountry())) {
                    UserDTO contact = userService.getUserDtoByPhone(phone, userEntity.getCountry());
                    /*if (contact == null) {
                        logger.warning("no contact with phone = " + phone);
                    } else if (addedPhones.contains(phone)) {
                        logger.warning("contact already in list. phone = " + phone);
                    } else if (contactService.areFriends(userEntity.getId(), contact.getId())) {
                        logger.warning("contacts already friends. phone = " + phone);
                    } else if (requestService.isThereAnOngoingRequest(userEntity.getId(), contact.getId())) {
                        logger.warning("request between users active. phone = " + phone);
                    } else {
                        addedPhones.add(phone);
                        ret.add(contact);
                    }//*/
                    //*
                    if (contact != null && !addedPhones.contains(phone)
                            && !contactService.areFriends(userEntity.getId(), contact.getId())
                            && !requestService.isThereAnOngoingRequest(userEntity.getId(), contact.getId())) {
                        addedPhones.add(phone);
                        ret.add(contact);
                    }//*/
                }
            }
            return new ResponseEntity<>(ret, HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }
}
