package restapi.controllers;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import restapi.json.MatchRequestDTO;
import restapi.location.LocationDTO;
import restapi.services.ContactService;
import restapi.services.LocationService;
import restapi.services.UserService;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.LoggedUser;
import restapi.utils.services.date.DateService;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
public class LocationController {

    @NonNull
    private final SimpMessagingTemplate messagingTemplate;
    @NonNull
    private final LocationService locationService;
    @NonNull
    private final ContactService contactService;
    @NonNull
    private final UserService userService;
    @NonNull
    private final DateService dateService;
    private final Logger logger = Logger.getLogger(LocationController.class.getName());

    /**
     * Updates user's location on the system and sends notification to all users involved in a match.
     *
     * @param locationDTO requested param with new location of the logged user
     */
    public void updateLocation(@RequestParam LocationDTO locationDTO) {
        try {
            locationService.saveLocation(locationDTO);
            long myID = userService.getIdByUsername(locationDTO.getUsername());
            HashMap<Long, Long> contacts = contactService.findActiveContacts(myID);
            HashMap<Long, MatchRequestDTO> contactsNearMe = locationService.getNearMe(myID, contacts, locationDTO);
            for (Map.Entry<Long, MatchRequestDTO> entry : contactsNearMe.entrySet()) {
                Long contactId = entry.getKey();
                MatchRequestDTO matchRequestDTO = entry.getValue();
                String user = userService.getUsernameById(contactId);
                messagingTemplate.convertAndSendToUser(user, "/queue/matches", matchRequestDTO);
                messagingTemplate.convertAndSendToUser(locationDTO.getUsername(), "/queue/matches",
                        new MatchRequestDTO(matchRequestDTO.getId(), userService.getDtoById(contactId), myID, matchRequestDTO.getDistanceRange(), dateService.nowToString(), matchRequestDTO.getState()));
                // firebaseService.sendMatch(user, locationDTO.getUsername(), matchRequestDTO.getDistanceRange());
            }
        } catch (CustomNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Update location of user and search for contacts nearby. Then it sends a message with the match to both users through websocket.
     * endpoint /location/update (PUT)
     *
     * @param user      Authenticated LoggedUser from token
     * @param latitude  Location latitude of user in grades
     * @param longitude Location longitude of user in grades
     */
    @Operation(
            summary = "Update location of user",
            description = "Update location of user and search for contacts nearby. Then it sends a message with the match to both users through websocket.",
            tags = {"Location"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "latitude", description = "Location latitude of user in grades", required = true, schema = @Schema(implementation = Double.class)),
                    @Parameter(name = "longitude", description = "Location longitude of user in grades", required = true, schema = @Schema(implementation = Double.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The server accepted and updated the location"),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/location/update")
    public void updateLocationAsync(@AuthenticationPrincipal final LoggedUser user,
                                    @RequestParam final double latitude,
                                    @RequestParam final double longitude) {
        logger.info("Location Update as PUT method with user: " + user.getUsername() + ", lat: " + latitude + " and lon: " + longitude);
        updateLocation(new LocationDTO(latitude, longitude, user.getUsername()));
    }

    /**
     * Updates user's location on the system and sends notification to all users involved in a match.
     * Meant for location update on background.
     * endpoint /location/update (POST)
     *
     * @param user Authenticated LoggedUser from token
     * @param body Contains Location in form of json: {lat: latitude, lon: longitude}
     */
    @Operation(
            summary = "Update location of user from background",
            description = "Update location of user and search for contacts nearby. Then it sends a message with the match to both users through websocket. \nMeant for location update on background.",
            tags = {"Location"},
            requestBody = @io.swagger.v3.oas.annotations.parameters.RequestBody(description = "Contains Location in form of json: {lat: latitude, lon: longitude}"),
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER)
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The server accepted and updated the location"),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PostMapping("/location/update")
    public void updateLocationSync(@AuthenticationPrincipal final LoggedUser user,
                                   @RequestBody List<? extends Map<String, Double>> body) {
        logger.info("Location Update as POST method with user: " + user.getUsername() + " and BODY: " + body.toString());
        Double lat = body.get(0).get("lat");
        Double lon = body.get(0).get("lon");
        updateLocation(new LocationDTO(lat, lon, user.getUsername()));
    }


}
