package restapi.controllers;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import restapi.crud.MatchEntity;
import restapi.crud.MatchRequestEntity;
import restapi.json.RestDictionary;
import restapi.json.RestList;
import restapi.json.RestMessage;
import restapi.json.RestResponse;
import restapi.services.MatchRequestService;
import restapi.services.MatchService;
import restapi.services.UserService;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.LoggedUser;
import restapi.utils.swagger.SwaggerActiveMatches;
import restapi.utils.swagger.SwaggerContact;

import java.util.List;
import java.util.function.Function;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@RequestMapping("/matches")
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
public class MatchController {
    @NonNull
    private final UserService userService;
    @NonNull
    private final MatchService matchService;
    @NonNull
    private final MatchRequestService matchRequestService;

    /**
     * Returns sent and received match requests with the details of the contact in the match, and a list of matches not requested
     * endpoint /matches/active
     *
     * @param user Authenticated LoggedUser from token
     * @return Lists of sent and received match requests and unrequested matches
     */
    @Operation(
            summary = "Get sent and received match requests + unrequested matches",
            description = "Returns sent and received match requests with the details of the contact in the match, and a list of matches not requested.",
            tags = {"Matches"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Lists of sent and received match requests and unrequested matches",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerActiveMatches.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/active")
    public ResponseEntity<RestResponse> getActiveMatches(@AuthenticationPrincipal final LoggedUser user) {
        try {
            long userId = userService.getIdByUsername(user.getUsername());
            // SENT
            List<MatchRequestEntity> requestList = matchRequestService.getIamSenderActiveMatchRequests(userId);
            RestList sentRequests = new RestList();
            for (MatchRequestEntity matchRequest : requestList) {
                sentRequests.add(matchRequestService.getIamSenderDtoFromEntity(matchRequest));
            }
            // RECEIVED
            requestList = matchRequestService.getReceivedActiveMatchRequests(userId);
            RestList receivedRequests = new RestList();
            for (MatchRequestEntity matchRequest : requestList) {
                receivedRequests.add(matchRequestService.getIamReceiverDtoFromEntity(matchRequest));
            }
            // ACTIVE
            List<MatchEntity> activeList = matchService.getActiveMatches(userId);
            RestList activeRequests = new RestList();
            for (MatchEntity matchEntity : activeList) {
                activeRequests.add(matchRequestService.getDtoFromMatchEntityAndUserId(matchEntity, userId, "unrequested"));
            }
            RestDictionary ret = new RestDictionary();
            ret.put("sent", sentRequests);
            ret.put("received", receivedRequests);
            ret.put("active", activeRequests);
            return new ResponseEntity<>(ret, HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }

    /**
     * Get user's match requests that have been accepted by both users
     * endpoint /matches/fulfilled
     *
     * @param user Authenticated LoggedUser from token
     * @return Lists of fulfilled match requests
     */
    @Operation(
            summary = "Get user's match requests that have been accepted by both users",
            description = "Returns user's match requests that have been accepted by both users.",
            tags = {"Matches"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Lists of fulfilled match requests",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerActiveMatches.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/fulfilled")
    public ResponseEntity<RestResponse> getFulfilledMatches(@AuthenticationPrincipal final LoggedUser user) {
        try {
            long userId = userService.getIdByUsername(user.getUsername());
            List<MatchRequestEntity> matchList = matchRequestService.getAcceptedMatchRequests(userId);
            RestList matches = new RestList();
            for (MatchRequestEntity matchEntity : matchList) {
                matches.add(matchRequestService.getDtoFromEntityAndUserId(matchEntity, userId));
            }
            RestDictionary ret = new RestDictionary();
            ret.put("matches", matches);
            return new ResponseEntity<>(ret, HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }
}
