package restapi.controllers;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import restapi.crud.MatchEntity;
import restapi.crud.MatchRequestEntity;
import restapi.json.MatchRequestDTO;
import restapi.json.RestMessage;
import restapi.json.RestResponse;
import restapi.services.MatchRequestService;
import restapi.services.MatchService;
import restapi.services.UserService;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomInternalErrorException;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.LoggedUser;

import java.util.logging.Logger;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@RequestMapping("/match")
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
public class MatchRequestController {
    @NonNull
    private final MatchRequestService matchRequestService;
    @NonNull
    private final UserService userService;
    @NonNull
    private final MatchService matchService;
    @NonNull
    private final SimpMessagingTemplate messagingTemplate;

    private final Logger logger = Logger.getLogger(MatchRequestController.class.getName());

    /**
     * Send new match request: Creates a match request for the given match id and sends it to contact
     * endpoint /match/send
     *
     * @param user    Authenticated LoggedUser from token
     * @param matchId Id of match the user wants to start an interaction for
     * @return Rest Message OK or Error 404, 409 or 500
     */
    @Operation(
            summary = "Send new match request",
            description = "Creates a match request for the given match id and sends it to contact",
            tags = {"Matches"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "matchId", description = "Id of match the user wants to start an interaction for", required = true, schema = @Schema(implementation = long.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The match request was successfully created and sent.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "500", description = "Internal Server Error: this match is between users that don't match with the given user.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PostMapping("/send")
    public ResponseEntity<RestResponse> postSendMatchRequest(@AuthenticationPrincipal final LoggedUser user,
                                                             @RequestParam final long matchId) {
        try {
            long userId = userService.getIdByUsername(user.getUsername());
            MatchEntity match = matchService.getMatchById(matchId);
            matchRequestService.addNewMatchRequest(userId, match);
            match = matchService.sendRequest(userId, match);
            MatchRequestDTO matchRequestDTO = matchRequestService.getDtoFromMatchEntityAndSenderId(match, userId, "active");
            messagingTemplate.convertAndSendToUser(userService.getUsernameById(matchRequestDTO.getOtherUser()), "/queue/matches", matchRequestDTO); //TODO
            return new ResponseEntity<>(new RestMessage((short) 200, "Match request registered"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        } catch (CustomInternalErrorException e) {
            logger.warning(e.getMessage());
            e.printStackTrace();
            return new ResponseEntity<>(new RestMessage((short) 500, e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * Delete a sent match request for the given match id if and only if the user sent it in the first place. Then contact gets notified.
     * endpoint /match/cancel
     *
     * @param user    Authenticated LoggedUser from token
     * @param matchId Id of match the user wants to cancel the request for
     * @return Rest Message OK or Error 404, 409 or 500
     */
    @Operation(
            summary = "Deletes sent match request",
            description = "Delete a sent match request for the given match id if and only if the user sent it in the first place. Then contact gets notified.",
            tags = {"Matches"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "matchId", description = "Id of match the user wants to cancel the request for", required = true, schema = @Schema(implementation = long.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The match request was successfully deleted.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "500", description = "Internal Server Error: this match is between users that don't match with the given user.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @DeleteMapping("/cancel")
    public ResponseEntity<RestResponse> postCancelMatchRequest(@AuthenticationPrincipal final LoggedUser user,
                                                               @RequestParam final long matchId) {
        try {
            long userId = userService.getIdByUsername(user.getUsername());
            MatchRequestEntity match = matchRequestService.cancelMatchRequest(matchId, userId);
            matchService.cancelRequest(userId, matchId);
            MatchRequestDTO matchRequestDTO = matchRequestService.getDtoFromEntityAndSenderId(match, userId);
            messagingTemplate.convertAndSendToUser(userService.getUsernameById(matchRequestDTO.getOtherUser()), "/queue/matches", matchRequestDTO);
            return new ResponseEntity<>(new RestMessage((short) 200, "Match request canceled"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        } catch (CustomInternalErrorException e) {
            logger.warning(e.getMessage());
            e.printStackTrace();
            return new ResponseEntity<>(new RestMessage((short) 500, e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * Accepts the received match request for the given match id if and only if the match request exists and the user is the receiver. Then contact gets notified.
     * endpoint /match/accept
     *
     * @param user    Authenticated LoggedUser from token
     * @param matchId Id of match the user wants to accept the request for
     * @return Rest Message OK or Error 404, 409 or 500
     */
    @Operation(
            summary = "Accept received match request",
            description = "Accepts the received match request for the given match id if and only if the match request exists and the user is the receiver. Then contact gets notified.",
            tags = {"Matches"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "matchId", description = "Id of match the user wants to accept the request for", required = true, schema = @Schema(implementation = long.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The match request was successfully deleted.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "500", description = "Internal Server Error: this match is between users that don't match with the given user.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/accept")
    public ResponseEntity<RestResponse> postAcceptMatchRequest(@AuthenticationPrincipal final LoggedUser user,
                                                               @RequestParam final long matchId) {
        try {
            long userId = userService.getIdByUsername(user.getUsername());
            MatchRequestEntity match = matchRequestService.acceptMatchRequest(matchId, userId);
            MatchRequestDTO matchRequestDTO = matchRequestService.getDtoFromEntityAndSenderId(match, userId);
            messagingTemplate.convertAndSendToUser(userService.getUsernameById(matchRequestDTO.getOtherUser()), "/queue/matches", matchRequestDTO);
            return new ResponseEntity<>(new RestMessage((short) 200, "Match request accepted"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    /*
    @PostMapping("/omit")
    public ResponseEntity<RestResponse> postOmitMatchRequest(@AuthenticationPrincipal final LoggedUser user,
                                                               @RequestParam final long matchId) {
        // TODO unimplemented
        try {
            return new ResponseEntity<>(new RestMessage((short) 200, "Match request registered"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    @PostMapping("/reject")
    public ResponseEntity<RestResponse> postRejectMatchRequest(@AuthenticationPrincipal final LoggedUser user,
                                                               @RequestParam final long matchId) {
        // TODO unimplemented
        try {
            return new ResponseEntity<>(new RestMessage((short) 200, "Match request registered"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }
    //*/

}
