package restapi.controllers;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import restapi.crud.RequestEntity;
import restapi.json.*;
import restapi.services.RequestService;
import restapi.services.UserService;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.LoggedUser;
import restapi.utils.swagger.SwaggerFriendshipRequests;

import java.util.ArrayList;
import java.util.List;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@RequestMapping("/requests")
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
public class RequestController {
    @NonNull
    private final RequestService requestService;
    @NonNull
    private final UserService userService;

    /**
     * @param username
     * @param option
     * @return
     */
    private ResponseEntity<RestResponse> getRequests(String username, int option) {
        try {
            long myID = userService.getIdByUsername(username);
            RestList sent = new RestList();
            RestList received = new RestList();
            List<RequestEntity> requests;
            switch (option) {
                case 0:
                    requests = requestService.getActiveRequests(myID);
                    requests.addAll(requestService.getRejectedRequests(myID));
                    break;
                case 1:
                    requests = requestService.getAcceptedRequests(myID);
                    break;
                default:
                    requests = new ArrayList<>();
            }
            for (RequestEntity request : requests) {
                if (request.getSenderID() == myID) {
                    UserDTO userDTO = userService.getDtoFromUserId(request.getReceiverID());
                    sent.add(new RequestDTO(request.getState(), userDTO));
                } else if (request.getReceiverID() == myID) {
                    UserDTO userDTO = userService.getDtoFromUserId(request.getSenderID());
                    received.add(new RequestDTO(request.getState(), userDTO));
                }
            }
            RestDictionary ret = new RestDictionary();
            ret.put("sent", sent);
            ret.put("received", received);
            return new ResponseEntity<>(ret, HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        }
    }

    /**
     * Returns a list of sent friendship requests and a list of received friendship requests. The requests that have been rejected by the other user are currently included in the sent list.
     * endpoint /requests/mine
     *
     * @param user Authenticated LoggedUser from token
     * @return List of friendship request in form of Request DTO
     */
    @Operation(
            summary = "Get all user's active friendship requests",
            description = "Returns a list of sent friendship requests and a list of received friendship requests. The requests that have been rejected by the other user are currently included in the sent list.",
            tags = {"Friendship Requests"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The friendship request was successfully created and sent.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerFriendshipRequests.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/mine")
    public ResponseEntity<RestResponse> getRequests(@AuthenticationPrincipal final LoggedUser user) {
        return getRequests(user.getUsername(), 0);
    }

    /**
     * Returns a list of accepted friendship requests and a list of received friendship requests. The requests that have been rejected by the other user are currently included in the sent list, but the rejected received requests are not returned.
     * endpoint /requests/accepted
     *
     * @param user Authenticated LoggedUser from token
     * @return List of friendship request in form of Request DTO
     */
    @Operation(
            summary = "Get user's accepted friendship requests",
            description = "Returns a list of accepted friendship requests and a list of received friendship requests. The requests that have been rejected by the other user are currently included in the sent list, but the rejected received requests are not returned.",
            tags = {"Friendship Requests"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The friendship request was successfully created and sent.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerFriendshipRequests.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/accepted")
    public ResponseEntity<RestResponse> getAcceptedRequests(@AuthenticationPrincipal final LoggedUser user) {
        return getRequests(user.getUsername(), 1);
    }

    /**
     * Creates a friendship request between logged user and user matching username.
     * endpoint /requests/new
     *
     * @param user     Authenticated LoggedUser from token
     * @param username Username of the user we want to send the friendship request to
     * @return Rest Message OK or Error 404, 409 or 500
     */
    @Operation(
            summary = "Send a friendship request to user",
            description = "Creates a friendship request between logged user and user matching username.",
            tags = {"Friendship Requests"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "username", description = "Username of the user we want to send the friendship request to.", required = true, schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The friendship request was successfully created and sent.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PostMapping("/new")
    public ResponseEntity<RestResponse> newRequest(@AuthenticationPrincipal final LoggedUser user,
                                                   @RequestParam final String username) {
        try {
            long sender = userService.getIdByUsername(user.getUsername());
            long receiver = userService.getIdByUsername(username);
            requestService.addRequest(sender, receiver);
            return new ResponseEntity<>(new RestMessage((short) 200, "Request Sent"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    /**
     * Accepts the received friendship request from the user matching the given username, if and only if the friendship request exists and the user is the receiver. Then Creates the Contact relation between both users.
     *
     * endpoint /requests/accept
     *
     * @param user     Authenticated LoggedUser from token
     * @param username Username of the user who send the friendship request we want to accept
     * @return Rest Message OK or Error 404, 409 or 500
     */
    @Operation(
            summary = "Accept a received friendship request",
            description = "Accepts the received friendship request from the user matching the given username, if and only if the friendship request exists and the user is the receiver. Then Creates the Contact relation between both users.",
            tags = {"Friendship Requests"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "username", description = "Username of the user who send the friendship request we want to accept.", required = true, schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The friendship request was successfully accepted.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/accept")
    public ResponseEntity<RestResponse> acceptRequest(@AuthenticationPrincipal final LoggedUser user,
                                                      @RequestParam final String username) {
        try {
            long sender = userService.getIdByUsername(username);
            long receiver = userService.getIdByUsername(user.getUsername());
            requestService.acceptRequest(sender, receiver);
            return new ResponseEntity<>(new RestMessage((short) 200, "Request Accepted"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    /**
     * Rejects the received friendship request from the user matching the given username, if and only if the friendship request exists and the user is the receiver
     *
     * endpoint /requests/reject
     *
     * @param user     Authenticated LoggedUser from token
     * @param username Username of the user who send the friendship request we want to reject
     * @return Rest Message OK or Error 404, 409 or 500
     */
    @Operation(
            summary = "Reject a received friendship request",
            description = "Rejects the received friendship request from the user matching the given username, if and only if the friendship request exists and the user is the receiver.",
            tags = {"Friendship Requests"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "username", description = "Username of the user who send the friendship request we want to reject.", required = true, schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The friendship request was successfully rejected.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/reject")
    public ResponseEntity<RestResponse> rejectRequest(@AuthenticationPrincipal final LoggedUser user,
                                                      @RequestParam final String username) {
        try {
            long sender = userService.getIdByUsername(username);
            long receiver = userService.getIdByUsername(user.getUsername());
            requestService.rejectRequest(sender, receiver);
            return new ResponseEntity<>(new RestMessage((short) 200, "Request Rejected"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    /**
     * Deletes the send friendship request to the user matching the given username, if and only if the friendship request exists and the user is the sender.
     * endpoint /requests/cancel
     *
     * @param user     Authenticated LoggedUser from token
     * @param username Username of the user who the friendship request we want to reject was sent to
     * @return Rest Message OK or Error 404, 409 or 500
     */
    @Operation(
            summary = "Delete a sent friendship request",
            description = "Deletes the send friendship request to the user matching the given username, if and only if the friendship request exists and the user is the sender.",
            tags = {"Friendship Requests"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "username", description = "Username of the user who the friendship request we want to reject was sent to.", required = true, schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The friendship request was successfully deleted.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @DeleteMapping("/cancel")
    public ResponseEntity<RestResponse> cancelRequest(@AuthenticationPrincipal final LoggedUser user,
                                                      @RequestParam final String username) {
        try {
            long sender = userService.getIdByUsername(user.getUsername());
            long receiver = userService.getIdByUsername(username);
            requestService.cancelRequest(sender, receiver);
            return new ResponseEntity<>(new RestMessage((short) 200, "Request Cancelled"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    /*
    @PostMapping("/seen")
    public ResponseEntity<RestResponse> closeRequest(@AuthenticationPrincipal final LoggedUser user,
                                                     @RequestParam final String username) {
        try {
            long sender = userService.getIdByUsername(user.getUsername());
            long receiver = userService.getIdByUsername(username);
            requestService.closeRequest(sender, receiver);
            return new ResponseEntity<>(new RestMessage((short) 200, "Request Seen"), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }
    */
}
