package restapi.controllers;

import java.util.Optional;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.data.repository.query.Param;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import restapi.crud.UserEntity;
import restapi.crud.UserRepository;
import restapi.json.RestMessage;
import restapi.json.RestResponse;
import restapi.json.Token;
import restapi.services.FirebaseService;
import restapi.services.UserService;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.UserAuthenticationService;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@RequestMapping("/public/users")
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
final class UsersControllerPublic {
    @NonNull
    private final UserAuthenticationService authentication;
    @NonNull
    private final UserRepository userRepository;
    @NonNull
    private final UserService userService;
    @NonNull
    private final FirebaseService authService;

    /**
     * Creates a user, logs in, and returns access token.
     * endpoint /public/users/signup
     *
     * @param username Username must be unique
     * @param password Username must be unique
     * @param authUID  Firebase authUID corresponding to a user logged with phone
     * @param email    Email must be unique
     * @param country  Country code. ex: "ES"
     * @return Authentication Token
     */
    @Operation(
            summary = "Sign Up",
            description = "Creates a user, logs in, and returns access token.",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "username", description = "Username must be unique", required = true, schema = @Schema(implementation = String.class)),
                    @Parameter(name = "password", description = "Password will be encrypted", required = true, schema = @Schema(implementation = String.class)),
                    @Parameter(name = "authUID", description = "Firebase authUID corresponding to a user logged with phone", schema = @Schema(implementation = String.class), in = ParameterIn.QUERY),
                    @Parameter(name = "email", description = "Email must be unique", schema = @Schema(implementation = String.class), in = ParameterIn.QUERY),
                    @Parameter(name = "country", description = "Country code. ex: \"ES\".", schema = @Schema(implementation = String.class), in = ParameterIn.QUERY)
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The user was successfully signed in.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = Token.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "There was a conflict with the given data and the operation could not be done. Error code meaning: (1: username, 2: email, 3: phone) already in use.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PostMapping("/signup")
    public ResponseEntity<RestResponse> signup(@RequestParam("username") final String username,
                                               @RequestParam("password") final String password,
                                               @Param("authUID") final String authUID,
                                               @Param("email") final String email,
                                               @Param("country") final String country) {
        if (userRepository.existsByUsername(username)) {
            return new ResponseEntity<>(new RestMessage((short) 1, "Username already in use."), HttpStatus.CONFLICT); // 409
        } else if (email != null && userRepository.existsByEmail(email)) {
            return new ResponseEntity<>(new RestMessage((short) 2, "Email already in use."), HttpStatus.CONFLICT); // 409
        } else if (authUID != null) {
            try {
                String phoneNum = authService.getValidPhoneFromFireAuth(authUID);
                userService.signup(username, password, phoneNum, email, country);
            } catch (CustomConflictException e) {
                return new ResponseEntity<>(new RestMessage((short) 3, e.getMessage()), HttpStatus.CONFLICT); // 409
            } catch (CustomNotFoundException e) {
                return new ResponseEntity<>(new RestMessage((short) 4, e.getMessage()), HttpStatus.NOT_FOUND); // 404
            }
        } else {
            userService.signup(username, password, null, email, country);
        }
        ResponseEntity<RestResponse> res = login(username, password);
        return new ResponseEntity<>(res.getBody(), HttpStatus.CREATED); // 201
    }

    /**
     * Login after verifying phone with Firebase. Returns Token.
     * endpoint /public/users/logintoken
     *
     * @param authUID Firebase authUID corresponding to a user logged with phone
     * @return Authentication Token
     */
    @Operation(
            summary = "Login after verifying phone with Firebase",
            description = "Login after verifying phone with Firebase. Returns Token.",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "authUID", description = "Firebase authUID corresponding to a user logged with phone", schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The user was successfully signed in.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = Token.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))})
            }
    )
    @PostMapping("/logintoken")
    public ResponseEntity<RestResponse> loginToken(@RequestParam("authUID") final String authUID) {
        try {
            String username = authService.getUsernameByFireAuth(authUID);
            Optional<String> res = authentication.login(username);
            if (res.isPresent()) {
                return new ResponseEntity<>(new Token(res.get()), HttpStatus.OK); // 200
            } else {
                return new ResponseEntity<>(new RestMessage((short) 401, "Something went wrong"), HttpStatus.UNAUTHORIZED); // 401
            }
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 4, e.getMessage()), HttpStatus.NOT_FOUND); // 404
        }
    }

    /**
     * Login with username and password. Returns Token.
     * endpoint /public/users/login
     *
     * @param username user's id
     * @param password Password must match with the one on the database
     * @return Authentication Token
     */
    @Operation(
            summary = "Login with username and password",
            description = "Login with username and password. Returns Token.",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "username", description = "Username", required = true, schema = @Schema(implementation = String.class)),
                    @Parameter(name = "password", description = "Password must match with the one on the database", required = true, schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The user was successfully signed in.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = Token.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Wrong password",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))})
            }
    )
    @PostMapping("/login")
    public ResponseEntity<RestResponse> loginUsername(@RequestParam("username") final String username,
                                                      @RequestParam("password") final String password) {
        if (userRepository.existsByUsername(username)) {
            return login(username, password);
        } else {
            return new ResponseEntity<>(new RestMessage((short) 1, "Invalid username"), HttpStatus.NOT_FOUND); // 404
        }
    }

    /**
     * Login with email and password. Returns Token.
     * endpoint /public/users/loginemail
     *
     * @param email    user's id
     * @param password Password must match with the one on the database
     * @return Authentication Token
     */
    @Operation(
            summary = "Login with email and password",
            description = "Login with email and password. Returns Token.",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "email", description = "Email", required = true, schema = @Schema(implementation = String.class)),
                    @Parameter(name = "password", description = "Password must match with the one on the database", required = true, schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The user was successfully signed in.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = Token.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Wrong password",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))})
            }
    )
    @PostMapping("/loginemail")
    public ResponseEntity<RestResponse> loginEmail(@RequestParam("email") final String email,
                                                   @RequestParam("password") final String password) {
        Optional<UserEntity> user = userRepository.findByEmail(email);
        return user.map(userEntity -> login(userEntity.getUsername(), password)).orElseGet(() ->
                new ResponseEntity<>(new RestMessage((short) 2, "Invalid email"), HttpStatus.NOT_FOUND)); // 404
    }

    /**
     * Login with phone and password. Returns Token.
     * endpoint /public/users
     *
     * @param phone    user's id
     * @param password Password must match with the one on the database
     * @return Authentication Token
     */
    @Operation(
            summary = "Login with phone and password",
            description = "Login with phone and password. Returns Token.",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "phone", description = "Phone", required = true, schema = @Schema(implementation = String.class)),
                    @Parameter(name = "password", description = "Password must match with the one on the database", required = true, schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "The user was successfully signed in.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = Token.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Wrong password",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))})
            }
    )
    @PostMapping("/loginphone")
    public ResponseEntity<RestResponse> loginPhone(@RequestParam("phone") final String phone,
                                                   @RequestParam("password") final String password) {
        Optional<UserEntity> user = userRepository.findByPhone(phone);
        return user.map(userEntity -> login(userEntity.getUsername(), password)).orElseGet(() ->
                new ResponseEntity<>(new RestMessage((short) 3, "Invalid phone"), HttpStatus.NOT_FOUND)); // 404
    }

    /**
     * Common function to internally make the login
     *
     * @param username username id
     * @param password Password must match with the one on the database
     * @return Authentication Token
     */
    public ResponseEntity<RestResponse> login(final String username, final String password) {
        Optional<String> res = authentication.login(username, password);
        if (res.isPresent()) {
            return new ResponseEntity<>(new Token(res.get()), HttpStatus.OK); // 200
        } else {
            return new ResponseEntity<>(new RestMessage((short) 401, "Invalid password"), HttpStatus.UNAUTHORIZED); // 401
        }
    }
}
