package restapi.controllers;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.data.repository.query.Param;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import restapi.crud.UserEntity;
import restapi.json.RestMessage;
import restapi.json.RestResponse;
import restapi.json.UserDTO;
import restapi.services.FirebaseService;
import restapi.services.UserService;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.security.auth.LoggedUser;
import restapi.utils.security.auth.UserAuthenticationService;
import restapi.utils.swagger.SwaggerFriendshipRequests;
import restapi.utils.swagger.SwaggerUser;

import static lombok.AccessLevel.PACKAGE;
import static lombok.AccessLevel.PRIVATE;

@RestController
@RequestMapping("/users")
@FieldDefaults(level = PRIVATE, makeFinal = true)
@AllArgsConstructor(access = PACKAGE)
final class UsersControllerSecured {
    @NonNull
    private final UserAuthenticationService authentication;
    @NonNull
    private final UserService userService;
    @NonNull
    private final FirebaseService authService;

    /**
     * Get user's details
     * endpoint /users/current
     *
     * @param user Authenticated LoggedUser from token
     * @return user's details
     */
    @Operation(
            summary = "Get user's details",
            description = "Return user's details",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Returns user's details",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = SwaggerUser.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @GetMapping("/current")
    public ResponseEntity<RestResponse> getCurrent(@AuthenticationPrincipal final LoggedUser user) {
        UserDTO u = userService.getDtoFromUserId(user.getUsername());
        return new ResponseEntity<>(u, HttpStatus.OK);
    }

    /**
     * Logout: Ends users session, and revokes permission to user's token
     * endpoint /users/logout
     *
     * @param user Authenticated LoggedUser from token
     * @return Rest Message OK
     */
    @Operation(
            summary = "Logout",
            description = "Ends users session, and revokes permission to user's token",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "User successfully logged out.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PostMapping("/logout")
    public ResponseEntity<RestResponse> logout(@AuthenticationPrincipal final LoggedUser user) {
        authentication.logout(user);
        return new ResponseEntity<>(new RestMessage((short) 200, "Logged Out"), HttpStatus.OK);
    }

    /**
     * Edit user's name and/or about
     * endpoint /users/editInfo
     *
     * @param user  Authenticated LoggedUser from token
     * @param name  Display name for the user
     * @param about Custom additional information
     * @return Rest Message OK or Error 404
     */
    @Operation(
            summary = "Edit user's name and/or about",
            description = "Edit user's name and/or about",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "name", description = "Display name for the user", schema = @Schema(implementation = String.class), in = ParameterIn.QUERY),
                    @Parameter(name = "about", description = "Custom additional information.", schema = @Schema(implementation = String.class), in = ParameterIn.QUERY)
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Information successfully updated.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/editInfo")
    public ResponseEntity<RestResponse> editInfo(@AuthenticationPrincipal final LoggedUser user,
                                                 //@Param("phone") final String phone,
                                                 //@Param("email") final String email,
                                                 //@Param("country") final String country,
                                                 @Param("name") final String name,
                                                 @Param("about") final String about) {
        try {
            UserEntity userEntity = userService.editUserInfo(user.getUsername(), null, null, name, about);
            return new ResponseEntity<>(userService.getDtoFromEntity(userEntity), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    /**
     * Edit user's phone number. Phone number must be previously verified by Firebase. Parameter must be the returned authUID from Firebase.
     * endpoint /users/editPhone
     *
     * @param user    Authenticated LoggedUser from token
     * @param authUID Firebase authUID corresponding to a user logged with phone
     * @return Rest Message OK or Error 404
     */
    @Operation(
            summary = "Edit user's phone number",
            description = "Edit user's phone number. Phone number must be previously verified by Firebase. Parameter must be the returned authUID from Firebase.",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "authUID", description = "Firebase authUID corresponding to a user logged with phone", schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Information successfully updated.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/editPhone")
    public ResponseEntity<RestResponse> changePhone(@AuthenticationPrincipal final LoggedUser user,
                                                    @RequestParam("authUID") final String authUID) {
        try {
            String newPhone = authService.getValidPhoneFromFireAuth(authUID);
            UserEntity userEntity = userService.editUserInfo(user.getUsername(), newPhone);
            return new ResponseEntity<>(userService.getDtoFromEntity(userEntity), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 4, e.getMessage()), HttpStatus.NOT_FOUND); // 404
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT); // 409
        }
    }

    /**
     * Change Password
     * endpoint /users/psw
     *
     * @param user        Authenticated LoggedUser from token
     * @param oldPassword Old Password must match the saved password
     * @param password    New password for the user
     * @return Rest Message OK or Error 404, 409 or 401
     */
    @Operation(
            summary = "Change Password",
            description = "Change Password",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "old", description = "Old Password must match the saved password", schema = @Schema(implementation = String.class)),
                    @Parameter(name = "password", description = "New password for the user", schema = @Schema(implementation = String.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Password successfully updated.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "409", description = "Old password does not match with password stored.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/psw")
    public ResponseEntity<RestResponse> changePassword(@AuthenticationPrincipal final LoggedUser user,
                                                       @RequestParam("old") final String oldPassword,
                                                       @RequestParam("password") final String password) {
        try {
            UserEntity userEntity = userService.changePassword(user.getUsername(), oldPassword, password);
            return new ResponseEntity<>(userService.getDtoFromEntity(userEntity), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }

    /**
     * Edit user's avatar's id.
     * endpoint /users/avatar
     *
     * @param user   Authenticated LoggedUser from token
     * @param avatar Avatar id
     * @return Rest Message OK or Error 404
     */
    @Operation(
            summary = "Edit user's avatar",
            description = "Edit user's avatar's id.",
            tags = {"User"},
            parameters = {
                    @Parameter(name = "Authorization", description = "Access Token", required = true, schema = @Schema(implementation = String.class), in = ParameterIn.HEADER),
                    @Parameter(name = "avatar", description = "Avatar id", schema = @Schema(implementation = int.class))
            },
            responses = {
                    @ApiResponse(responseCode = "200", description = "Avatar id successfully updated.",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "404", description = "Something has been not found in the db",
                            content = {@Content(mediaType = "application/json", schema = @Schema(implementation = RestMessage.class))}),
                    @ApiResponse(responseCode = "401", description = "Invalid Token")
            }
    )
    @PutMapping("/avatar")
    public ResponseEntity<RestResponse> changeAvatar(@AuthenticationPrincipal final LoggedUser user,
                                                       @RequestParam("avatar") final int avatar) {
        try {
            UserEntity userEntity = userService.changeAvatar(user.getUsername(), avatar);
            return new ResponseEntity<>(userService.getDtoFromEntity(userEntity), HttpStatus.OK);
        } catch (CustomNotFoundException e) {
            return new ResponseEntity<>(new RestMessage((short) 404, e.getMessage()), HttpStatus.NOT_FOUND);
        } catch (CustomConflictException e) {
            return new ResponseEntity<>(new RestMessage((short) 409, e.getMessage()), HttpStatus.CONFLICT);
        }
    }
}