package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import restapi.crud.ContactEntity;
import restapi.crud.ContactPreferencesEntity;
import restapi.crud.ContactPreferencesRepository;
import restapi.crud.ContactRepository;
import restapi.json.ContactDTO;
import restapi.json.RestList;
import restapi.utils.error.CustomNotFoundException;

import java.util.Optional;

@Service
@AllArgsConstructor
public class ContactPreferencesService {
    @NonNull
    private ContactRepository contactRepository;
    @NonNull
    private ContactPreferencesRepository contactPreferencesRepository;
    @NonNull
    private UserService userService;
    @NonNull
    private UserSessionService userSessionService;
    @NonNull
    private final LocationService locationService;

    /**
     * Set user's preferences for contact to default
     *
     * @param userId    User whose preferences we are setting
     * @param contactId Contact who we are setting the preferences for
     */
    public void addPreferences(long userId, long contactId) {
        ContactPreferencesEntity contactPreferences = new ContactPreferencesEntity();
        contactPreferences.setUserId(userId);
        contactPreferences.setContactId(contactId);
        contactPreferences.setParam(500L);
        contactPreferences.setBlocked(false);
        contactPreferencesRepository.saveAndFlush(contactPreferences);
    }

    /**
     * Finds the contact entity between two users and returns it or throws exception when not found
     *
     * @param user1 Id for user
     * @param user2 Id for user
     * @return The friendship as ContactEntity
     * @throws CustomNotFoundException When user are not friends
     */
    public ContactEntity findFriends(long user1, long user2) throws CustomNotFoundException {
        Optional<ContactEntity> contactOp = contactRepository.findByUserId1AndUserId2(user1, user2);
        if (contactOp.isPresent()) return contactOp.get();
        contactOp = contactRepository.findByUserId1AndUserId2(user2, user1);
        if (contactOp.isPresent()) return contactOp.get();
        throw new CustomNotFoundException("These users are not friends");
    }

    /**
     * Updates most restrictive range between users when needed
     *
     * @param userId    Id of user
     * @param contactId Id of contact
     * @param range     New parameter for range
     * @throws CustomNotFoundException When user not found
     */
    public void updateRange(long userId, long contactId, long range) throws CustomNotFoundException {
        ContactEntity contact = findFriends(userId, contactId);
        if (contact.getParam() > range) {
            contact.setParam(range);
            contactRepository.saveAndFlush(contact);
        } else {
            Optional<ContactPreferencesEntity> contactPrefOp = contactPreferencesRepository.findByUserIdAndContactId(contactId, userId);
            if (contactPrefOp.isPresent()) {
                Long contactRange = contactPrefOp.get().getParam();
                if (contactRange > range) {
                    contact.setParam(range);
                    contactRepository.saveAndFlush(contact);
                } else if (!contact.getParam().equals(contactRange)) { // if (contactRange < range) , contact.param is already updated.
                    contact.setParam(contactRange);
                    contactRepository.saveAndFlush(contact);
                }
                // find match when change param to longer range
                if (contact.getParam() != null && contact.getBlocked() != null && !contact.getBlocked() &&
                        userSessionService.isOnline(userService.getUser(contact.getUserId1())) &&
                        userSessionService.isOnline(userService.getUser(contact.getUserId2()))) {
                    locationService.sendMatch(userId, contactId, contact);
                }
            }
        }
    }

    /**
     * Updates most restrictive blocked between users when needed
     *
     * @param userId    Id of user
     * @param contactId Id of contact
     * @param blocked   New value (boolean) for blocked
     * @throws CustomNotFoundException When users are not friends
     */
    public void updateBlocked(long userId, long contactId, boolean blocked) throws CustomNotFoundException {
        ContactEntity contact = findFriends(userId, contactId);
        if (contact.getBlocked() != null && contact.getBlocked()) {
            if (!blocked) {
                Optional<ContactPreferencesEntity> contactPrefOp = contactPreferencesRepository.findByUserIdAndContactId(contactId, userId);
                if (contactPrefOp.isPresent()) {
                    boolean contactBlocked = contactPrefOp.get().isBlocked();
                    if (!contactBlocked) {
                        contact.setBlocked(false);
                        contactRepository.saveAndFlush(contact);
                    }
                }
            }
        } else if (blocked) {
            contact.setBlocked(true);
            contactRepository.saveAndFlush(contact);
        }
    }

    /**
     * Updates blocked on contact's configurations for contact. Updates most restrictive blocked between users when needed
     *
     * @param userId    Id of user
     * @param contactId Id of contact
     * @param param     New value (boolean) for blocked
     * @throws CustomNotFoundException When users are not friends
     */
    public void changeParam(long userId, long contactId, long param) throws CustomNotFoundException {
        Optional<ContactPreferencesEntity> pref = contactPreferencesRepository.findByUserIdAndContactId(userId, contactId);
        if (pref.isPresent()) {
            ContactPreferencesEntity contactPreferencesEntity = pref.get();
            contactPreferencesEntity.setParam(param);
            contactPreferencesRepository.saveAndFlush(contactPreferencesEntity);
            updateRange(userId, contactId, param);
        } else {
            throw new CustomNotFoundException("No contact with such id.");
        }
    }

    /**
     * Updates blocked on contact's configurations for contact. Updates most restrictive blocked between users when needed
     *
     * @param userId    Id of user
     * @param contactId Id of contact
     * @param blocked   New value (boolean) for blocked
     * @throws CustomNotFoundException When users are not friends
     */
    public void changeBlocked(long userId, long contactId, boolean blocked) throws CustomNotFoundException {
        Optional<ContactPreferencesEntity> pref = contactPreferencesRepository.findByUserIdAndContactId(userId, contactId);
        if (pref.isPresent()) {
            ContactPreferencesEntity contactPreferencesEntity = pref.get();
            contactPreferencesEntity.setBlocked(blocked);
            contactPreferencesRepository.saveAndFlush(contactPreferencesEntity);
            updateBlocked(userId, contactId, blocked);
        } else {
            throw new CustomNotFoundException("No contact with such id.");
        }
    }

    /**
     * Get all contacts' parameters and personal info
     *
     * @param userId Id of user
     * @return RestList with ContactDTOs
     */
    public RestList getContacts(long userId) {
        RestList ret = new RestList();
        contactPreferencesRepository.findAllByUserId(userId).forEach((temp) -> {
            try {
                ret.add(getContactDTOFromEntity(temp));
            } catch (CustomNotFoundException e) {
                e.printStackTrace();
            }
        });
        return ret;
    }

    /**
     * Transforms Contact Entity to Contact DTO
     *
     * @param entity Contact Entity
     * @return Contact DTO
     * @throws CustomNotFoundException When user in ContactEntity not found
     */
    public ContactDTO getContactDTOFromEntity(ContactPreferencesEntity entity) throws CustomNotFoundException {
        return new ContactDTO(userService.getDtoFromUserId(entity.getContactId()), entity.getParam(), entity.isBlocked());
    }

    /**
     * Returns Contact DTO corresponding to preferences of user for contact
     *
     * @param userId    Id of user
     * @param contactId Id of contact
     * @return Contact DTO
     * @throws CustomNotFoundException When users are no contacts or user in ContactEntity not found
     */
    public ContactDTO getContactDTO(long userId, long contactId) throws CustomNotFoundException {
        Optional<ContactPreferencesEntity> prefOp = contactPreferencesRepository.findByUserIdAndContactId(userId, contactId);
        if (prefOp.isPresent()) {
            ContactPreferencesEntity pref = prefOp.get();
            return new ContactDTO(userService.getDtoFromUserId(contactId), pref.getParam(), pref.isBlocked());
        } else {
            throw new CustomNotFoundException("No contact with such id.");
        }
    }
}
