package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import restapi.crud.ContactEntity;
import restapi.crud.ContactRepository;
import restapi.utils.services.date.DateService;
import restapi.utils.error.CustomNotFoundException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

@Service
@AllArgsConstructor
public class ContactService {
    @NonNull
    private ContactRepository contactRepository;
    @NonNull
    private ContactPreferencesService contactPreferencesService;
    @NonNull
    private UserSessionService userSessionService;
    @NonNull
    private UserService userService;
    @NonNull
    private DateService dateService;

    /**
     * Get a list of user's contacts' ids
     *
     * @param id Id of user to retrieve the contacts of
     * @return list of user's contacts' ids
     */
    public List<Long> findUsersContacts(long id) {
        List<ContactEntity> contacts = contactRepository.findAllByUserId1OrUserId2(id, id);
        ArrayList<Long> ret = new ArrayList<>();
        for (ContactEntity contactEntity : contacts) {
            if (contactEntity.getUserId1() == id) {
                ret.add(contactEntity.getUserId2());
            } else {
                ret.add(contactEntity.getUserId1());
            }
        }
        return ret;
    }

    /**
     * Find Contact Entity between user 1 and user 2
     *
     * @param user1 Id of user 1
     * @param user2 Id of user 2
     * @return Optional of ContactEntity
     */
    public Optional<ContactEntity> findContact(long user1, long user2) {
        return contactRepository.findByUserId1AndUserId2(user1, user2);
    }

    /**
     * Returns True if users are friends, False otherwise
     *
     * @param user1 Id of user 1
     * @param user2 Id of user 2
     * @return True if users are friends, False otherwise
     */
    public boolean areFriends(long user1, long user2) {
        return findContact(user1, user2).isPresent() || findContact(user2, user1).isPresent();
    }

    public void addContact(long sender, long receiver) {
        ContactEntity contact = new ContactEntity();
        contact.setUserId1(sender);
        contact.setUserId2(receiver);
        contact.setParam(500L);
        contact.setBlocked(false);
        contact.setSince(dateService.nowToString());
        contactRepository.saveAndFlush(contact);
        contactPreferencesService.addPreferences(sender, receiver);
        contactPreferencesService.addPreferences(receiver, sender);
    }

    /**
     * Return contacts that don't have me blocked, that I don't have blocked, and that are online.
     *
     * @param userId Current user to find contacts of
     * @return HashMap ( CONTACT_ID : PARAM )
     * @throws CustomNotFoundException When contact not found
     */
    public HashMap<Long, Long> findActiveContacts(long userId) throws CustomNotFoundException {
        List<ContactEntity> contacts = contactRepository.findAllByUserId1OrUserId2(userId, userId);
        HashMap<Long, Long> contactParams = new HashMap<>();
        for (ContactEntity contactEntity : contacts) {
            if (contactEntity.getParam() != null && contactEntity.getBlocked() != null && !contactEntity.getBlocked()) {
                if (contactEntity.getUserId1() == userId) {
                    if (userSessionService.isOnline(userService.getUser(contactEntity.getUserId2()))) {
                        contactParams.put(contactEntity.getUserId2(), contactEntity.getParam());
                    }
                } else {
                    if (userSessionService.isOnline(userService.getUser(contactEntity.getUserId1()))) {
                        contactParams.put(contactEntity.getUserId1(), contactEntity.getParam());
                    }
                }
            }
        }
        return contactParams;
    }
}
