package restapi.services;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseAuthException;
import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import restapi.crud.UserEntity;
import restapi.crud.UserRepository;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomNotFoundException;

import java.util.Optional;

@Service
@AllArgsConstructor
public class FirebaseService {

    @NonNull
    private final UserRepository userRepository;

    /**
     * Retrieves phone number corresponding to auth UID from Firebase
     *
     * @param authUID Firebase authUID corresponding to a user logged with phone
     * @return Phone number in format String
     * @throws CustomNotFoundException When Firebase rejects the authUID
     */
    private String getPhoneFromFireAuth(String authUID) throws CustomNotFoundException {
        try {
            return FirebaseAuth.getInstance().getUser(authUID).getPhoneNumber();
        } catch (FirebaseAuthException e) {
            throw new CustomNotFoundException("Invalid firebase auth UID token: " + e);
        }
    }

    /**
     * Returns Username with phone number corresponding to the authUID on Firebase
     *
     * @param authUID Firebase authUID corresponding to a user logged with phone
     * @return Username
     * @throws CustomNotFoundException When Firebase rejects the authUID or Phone Number not found in database
     */
    public String getUsernameByFireAuth(String authUID) throws CustomNotFoundException {
        String phoneNum = getPhoneFromFireAuth(authUID);
        Optional<UserEntity> user = userRepository.findByPhone(phoneNum);
        if (!user.isPresent()) {
            throw new CustomNotFoundException("Invalid auth");
        }
        return user.get().getUsername();
    }

    /**
     * Returns phone number corresponding to the authUID on Firebase if the phone is not already used by a user
     *
     * @param authUID Firebase authUID corresponding to a user logged with phone
     * @return Phone Number in format String if it is not on the database
     * @throws CustomNotFoundException When Firebase rejects the authUID
     * @throws CustomConflictException When Phone Number already in database
     */
    public String getValidPhoneFromFireAuth(String authUID) throws CustomNotFoundException, CustomConflictException {
        String phoneNum = getPhoneFromFireAuth(authUID);
        boolean repeatedPhone = userRepository.existsByPhone(phoneNum);
        if (!repeatedPhone) {
            return phoneNum;
        } else {
            throw new CustomConflictException("Phone " + phoneNum + " already in use by another user");
        }
    }

}
