package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.stereotype.Service;
import restapi.crud.ContactEntity;
import restapi.crud.LocationEntity;
import restapi.crud.LocationRepository;
import restapi.json.MatchRequestDTO;
import restapi.location.LocationDTO;
import restapi.utils.services.date.DateService;
import restapi.utils.error.CustomNotFoundException;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.logging.Logger;

@Service
@AllArgsConstructor
public class LocationService {

    @NonNull
    private final UserService userService;
    @NonNull
    private final DateService dateService;
    @NonNull
    private final LocationRepository locationRepository;
    @NonNull
    private final SimpMessagingTemplate messagingTemplate;
    @NonNull
    private final MatchService matchService;
    private final Logger logger = Logger.getLogger(LocationService.class.getName());

    /**
     * Calculate distance between two points in latitude and longitude taking
     * into account height difference. If you are not interested in height
     * difference pass 0.0. Uses Haversine method as its base.
     * <p>
     * lat1, lon1 Start point lat2, lon2 End point el1 Start altitude in meters
     * el2 End altitude in meters
     *
     * @param lat1 Start point
     * @param lon1 Start point
     * @param el1  Start altitude in meters
     * @param lat2 End point
     * @param lon2 End point
     * @param el2  End altitude in meters
     * @return Distance in Meters
     */
    public static double distance(double lat1, double lon1, double el1,
                                  double lat2, double lon2, double el2) {
        final int R = 6371; // Radius of the earth

        double latDistance = Math.toRadians(lat2 - lat1);
        double lonDistance = Math.toRadians(lon2 - lon1);
        double a = Math.sin(latDistance / 2) * Math.sin(latDistance / 2)
                + Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2))
                * Math.sin(lonDistance / 2) * Math.sin(lonDistance / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        double distance = R * c * 1000; // convert to meters

        double height = el1 - el2;

        distance = Math.sqrt(Math.pow(distance, 2) + Math.pow(height, 2));

        return distance;
    }

    /**
     * Returns if two users are close enough to send notification
     *
     * @param loc1        LocationDTO of user 1
     * @param loc2        LocationDTO of user 2
     * @param restriction maximum distance between users to consider them near each other
     * @return if two users are close enough to send notification
     */
    protected boolean compareLocations(@NonNull LocationDTO loc1, @NonNull LocationDTO loc2, long restriction) {
        double r = distance(loc1.getLatitude(), loc1.getLongitude(), 0, loc2.getLatitude(), loc2.getLongitude(), 0);
        return r < restriction;
    }

    /**
     * Returns a map of contacts that are near the user and made from the map passed by parameter
     *
     * @param userId
     * @param contacts
     * @param locationDTO
     * @return
     * @throws CustomNotFoundException
     */
    public HashMap<Long, MatchRequestDTO> getNearMe(long userId, HashMap<Long, Long> contacts, LocationDTO locationDTO) throws CustomNotFoundException {
        HashMap<Long, MatchRequestDTO> matches = new HashMap<>();
        for (Map.Entry<Long, Long> entry : contacts.entrySet()) {
            long contactId = entry.getKey();
            long param = entry.getValue();
            if (!matchService.isThereARecentMatchBetweenUsers(userId, contactId)) { // TODO descomentar: comentar si vull fer proves
                LocationDTO loc = getLocation(contactId);
                if (loc != null && compareLocations(locationDTO, loc, param)) {
                    long matchId = matchService.saveMatch(userId, contactId, param, locationDTO.getLatitude(), locationDTO.getLongitude());
                    matches.put(entry.getKey(), new MatchRequestDTO(matchId, userService.getDtoFromUserId(userId), contactId, param, dateService.nowToString(), "unrequested"));
                }
            }
        }
        return matches;
    }

    /**
     * Get location saved on the database for user with id = id
     *
     * @param id
     * @return
     */
    public @Nullable LocationDTO getLocation(long id) {
        Optional<LocationEntity> loc = locationRepository.findByUserId(id);
        return loc.map(locationEntity -> new LocationDTO(locationEntity.getLatitude(), locationEntity.getLongitude(), String.valueOf(id))).orElse(null);
    }

    /**
     * Saves the location in the database
     *
     * @param locationDTO
     * @throws CustomNotFoundException
     */
    public void saveLocation(@NonNull LocationDTO locationDTO) throws CustomNotFoundException {
        long userID = userService.getIdByUsername(locationDTO.getUsername());
        Optional<LocationEntity> locOp = locationRepository.findByUserId(userID);
        LocationEntity loc;
        if (locOp.isPresent()) {
            loc = locOp.get();
            loc.setLatitude(locationDTO.getLatitude());
            loc.setLongitude(locationDTO.getLongitude());
            loc.setDateTime(dateService.nowToString());
        } else {
            loc = LocationEntity.builder()
                    .userId(userID)
                    .latitude(locationDTO.getLatitude())
                    .longitude(locationDTO.getLongitude())
                    .dateTime(dateService.nowToString()).build();
        }
        locationRepository.saveAndFlush(loc);
    }

    /**
     * If there is a recent match between users passed by parameter, it sends it to both users via websocket
     *
     * @param userId
     * @param contactId
     * @param contact
     * @throws CustomNotFoundException
     */
    public void sendMatch(long userId, long contactId, ContactEntity contact) throws CustomNotFoundException {
        if (!matchService.isThereARecentMatchBetweenUsers(userId, contactId)) {
            LocationDTO uLoc = getLocation(userId);
            LocationDTO cLoc = getLocation(contactId);
            if (uLoc != null && cLoc != null && compareLocations(uLoc, cLoc, contact.getParam())) {
                long matchId = matchService.saveMatch(userId, contactId, contact.getParam(), uLoc.getLatitude(), uLoc.getLongitude());
                MatchRequestDTO match = new MatchRequestDTO(matchId, userService.getDtoFromUserId(userId), contactId, contact.getParam(), dateService.nowToString(), "active");
                String user = userService.getUsernameById(userId);
                messagingTemplate.convertAndSendToUser(user, "/queue/matches", match);
                user = userService.getUsernameById(contactId);
                messagingTemplate.convertAndSendToUser(user, "/queue/matches", match);
            }
        }
    }

    /**
     * Send match to user via websocket
     *
     * @param username
     * @param match
     */
    public void sendChange(String username, MatchRequestDTO match) {
        messagingTemplate.convertAndSendToUser(username, "/queue/matches", match);
    }
}
