package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import restapi.crud.MatchEntity;
import restapi.crud.MatchRequestEntity;
import restapi.crud.MatchRequestRepository;
import restapi.json.MatchRequestDTO;
import restapi.utils.services.date.DateService;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomNotFoundException;

import java.util.List;
import java.util.Optional;

@Service
@AllArgsConstructor
public class MatchRequestService {
    @NonNull
    private DateService dateService;
    @NonNull
    private MatchRequestRepository matchRequestRepository;
    @NonNull
    private UserService userService;

    /**
     * Creates new MatchRequestEntity with sender userId, receiver other user in matchEntity, and matchEntity's id and saves it to database.
     *
     * @param userId      sender
     * @param matchEntity match info related to connection
     * @throws CustomConflictException when there is already an ongoing request with this match
     */
    public void addNewMatchRequest(long userId, MatchEntity matchEntity) throws CustomConflictException {
        if (matchRequestRepository.findByMatchId(matchEntity.getId()).isPresent()) {
            throw new CustomConflictException("There is already a match request for this request");
        }
        MatchRequestEntity matchRequestEntity = new MatchRequestEntity();
        matchRequestEntity.setMatchId(matchEntity.getId());
        matchRequestEntity.setSender(userId);
        matchRequestEntity.setReceiver(matchEntity.getUserId1() == userId ? matchEntity.getUserId2() : matchEntity.getUserId1());
        matchRequestEntity.setDatetime(dateService.nowToString());
        matchRequestEntity.setParam(matchEntity.getParam());
        matchRequestEntity.setState("active");
        matchRequestRepository.saveAndFlush(matchRequestEntity);
    }

    /**
     * Get match request the user is in (as sender or receiver)
     *
     * @param userId
     * @return
     */
    public List<MatchRequestEntity> getMatchRequests(long userId) {
        return matchRequestRepository.findAllBySenderOrReceiver(userId, userId);
    }

    /**
     * Returns a list with all the matches more recent than 60 min from the lis passed by parameter
     *
     * @param matches
     * @return
     */
    private List<MatchRequestEntity> filterOld(List<MatchRequestEntity> matches) {
        if (!matches.isEmpty())
            matches.removeIf(matchRequest -> dateService.isOlderThan(matchRequest.getDatetime(), 60));
        return matches;
    }

    /**
     * Returns a list of matches that have user as sender
     *
     * @param userId
     * @return
     */
    public List<MatchRequestEntity> getIamSenderMatchRequests(long userId) {
        return filterOld(matchRequestRepository.findAllBySender(userId));
    }

    /**
     * Returns a list of matches that have user as receiver
     *
     * @param userId
     * @return
     */
    public List<MatchRequestEntity> getReceivedMatchRequests(long userId) {
        return filterOld(matchRequestRepository.findAllByReceiver(userId));
    }

    /**
     * Returns a list of matches that have user as sender and are active
     *
     * @param userId
     * @return
     */
    public List<MatchRequestEntity> getIamSenderActiveMatchRequests(long userId) {
        return filterOld(matchRequestRepository.findAllBySenderAndState(userId, "active"));
    }

    /**
     * Returns a list of matches that have user as receiver and are active
     *
     * @param userId
     * @return
     */
    public List<MatchRequestEntity> getReceivedActiveMatchRequests(long userId) {
        return filterOld(matchRequestRepository.findAllByReceiverAndState(userId, "active"));
    }

    /**
     * Translates MatchRequestEntity into MatchRequestDTO with user == sender and contact == receiver
     *
     * @param entity
     * @return
     * @throws CustomNotFoundException
     */
    public MatchRequestDTO getIamSenderDtoFromEntity(MatchRequestEntity entity) throws CustomNotFoundException {
        return new MatchRequestDTO(entity.getMatchId(), userService.getDtoFromUserId(entity.getReceiver()), entity.getSender(), entity.getParam(), entity.getDatetime(), entity.getState());
    }

    /**
     * Translates MatchRequestEntity into MatchRequestDTO with user == receiver and contact == sender
     *
     * @param entity
     * @return
     * @throws CustomNotFoundException
     */
    public MatchRequestDTO getIamReceiverDtoFromEntity(MatchRequestEntity entity) throws CustomNotFoundException {
        return new MatchRequestDTO(entity.getMatchId(), userService.getDtoFromUserId(entity.getSender()), entity.getReceiver(), entity.getParam(), entity.getDatetime(), entity.getState());
    }

    /**
     * Constructs MatchRequestDTO with matchId from match, contact = user in match id != userId, and state == state
     *
     * @param match
     * @param userId
     * @param state
     * @return
     * @throws CustomNotFoundException
     */
    public MatchRequestDTO getDtoFromMatchEntityAndUserId(MatchEntity match, long userId, String state) throws CustomNotFoundException {
        long contactId = match.getUserId1() == userId ? match.getUserId2() : match.getUserId1();
        long id = match.getUserId1() == userId ? match.getUserId1() : match.getUserId2();
        return new MatchRequestDTO(match.getId(), userService.getDtoFromUserId(contactId), id, match.getParam(), match.getDatetime(), state);
    }

    /**
     * Constructs MatchRequestDTO with matchId from match, contact = user in match with id == userId, and state == state
     *
     * @param match
     * @param userId
     * @param state
     * @return
     * @throws CustomNotFoundException
     */
    public MatchRequestDTO getDtoFromMatchEntityAndSenderId(MatchEntity match, long userId, String state) throws CustomNotFoundException {
        long contactId = match.getUserId1() == userId ? match.getUserId1() : match.getUserId2();
        long id = match.getUserId1() == userId ? match.getUserId2() : match.getUserId1();
        return new MatchRequestDTO(match.getId(), userService.getDtoFromUserId(contactId), id, match.getParam(), match.getDatetime(), state);
    }

    /**
     * Translates MatchRequestEntity to MatchRequestDTO with userId as user (not contact)
     *
     * @param matchRequest
     * @param userId
     * @return
     * @throws CustomNotFoundException
     */
    public MatchRequestDTO getDtoFromEntityAndUserId(MatchRequestEntity matchRequest, long userId) throws CustomNotFoundException {
        long contactId = (matchRequest.getSender() == userId) ? matchRequest.getReceiver() : matchRequest.getSender();
        long id = (matchRequest.getSender() == userId) ? matchRequest.getSender() : matchRequest.getReceiver();
        return new MatchRequestDTO(matchRequest.getMatchId(), userService.getDtoFromUserId(contactId), id, matchRequest.getParam(), matchRequest.getDatetime(), matchRequest.getState());
    }

    /**
     * Translates MatchRequestEntity to MatchRequestDTO with userId as contact (not user)
     *
     * @param matchRequest
     * @param userId
     * @return
     * @throws CustomNotFoundException
     */
    public MatchRequestDTO getDtoFromEntityAndSenderId(MatchRequestEntity matchRequest, long userId) throws CustomNotFoundException {
        long contactId = (matchRequest.getSender() == userId) ? matchRequest.getSender() : matchRequest.getReceiver();
        long id = (matchRequest.getSender() == userId) ? matchRequest.getReceiver() : matchRequest.getSender();
        return new MatchRequestDTO(matchRequest.getMatchId(), userService.getDtoFromUserId(contactId), id, matchRequest.getParam(), matchRequest.getDatetime(), matchRequest.getState());
    }

    /**
     * Returns list of accepted matches for user whit userId
     *
     * @param userId
     * @return
     */
    public List<MatchRequestEntity> getAcceptedMatchRequests(long userId) {
        return matchRequestRepository.findAllByUserIDAndState(userId, "accepted");
    }

    /**
     * Get MatchRequestEntity By Match Id
     *
     * @param matchId
     * @return
     * @throws CustomNotFoundException
     */
    public MatchRequestEntity getByMatchId(long matchId) throws CustomNotFoundException {
        Optional<MatchRequestEntity> matchOp = matchRequestRepository.findByMatchId(matchId);
        if (matchOp.isPresent()) {
            return matchOp.get();
        } else throw new CustomNotFoundException("No match request with this match id.");
    }

    /**
     * Accepts match request if userId is receiver
     *
     * @param matchId
     * @param userId
     * @return
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public MatchRequestEntity acceptMatchRequest(long matchId, long userId) throws CustomNotFoundException, CustomConflictException {
        MatchRequestEntity entity = getByMatchId(matchId);
        if (entity.getReceiver() != userId)
            throw new CustomConflictException("This user is not the receiver of this match request thus it cannot be accepted");
        entity.setState("accepted");
        matchRequestRepository.saveAndFlush(entity);
        return entity;
    }

    /**
     * Cancels match request if userId is sender
     *
     * @param matchId
     * @param userId
     * @return
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public MatchRequestEntity cancelMatchRequest(long matchId, long userId) throws CustomNotFoundException, CustomConflictException {
        MatchRequestEntity entity = getByMatchId(matchId);
        if (entity.getSender() != userId)
            throw new CustomConflictException("This user is not the sender of this match request thus it cannot be cancelled");
        matchRequestRepository.delete(entity);
        return entity;
    }
}
