package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import restapi.crud.MatchEntity;
import restapi.crud.MatchRepository;
import restapi.location.LocationMatch;
import restapi.utils.services.date.DateService;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomInternalErrorException;
import restapi.utils.error.CustomNotFoundException;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@AllArgsConstructor
public class MatchService {
    @NonNull
    private DateService dateService;
    @NonNull
    private MatchRepository matchRepository;
    @NonNull
    private UserService userService;

    /**
     * Returns list of active matches between u1 and u2
     *
     * @param u1
     * @param u2
     * @return
     */
    public List<MatchEntity> getRecentMatchBetweenUsers(long u1, long u2) {
        List<MatchEntity> op1 = matchRepository.findAllByUserId1AndUserId2AndOutdated(u1, u2, false);
        List<MatchEntity> op2 = matchRepository.findAllByUserId1AndUserId2AndOutdated(u2, u1, false);
        op1.addAll(op2);
        return op1;
        /*if (op1.isEmpty()){
            throw new CustomNotFoundException("no match");
        } else {
        }//*/
    }

    /**
     * Returns true if there is a match between users
     *
     * @param u1
     * @param u2
     * @return
     */
    public boolean isThereARecentMatchBetweenUsers(long u1, long u2) {
        List<MatchEntity> recent = getRecentMatchBetweenUsers(u1, u2);
        int i = recent.size();
        for (MatchEntity matchEntity : recent) {
            if (dateService.isOlderThan(matchEntity.getDatetime(), 60)) { // TO DO change time now 1 min to 60 or sth
                matchEntity.setOutdated(true);
                matchRepository.saveAndFlush(matchEntity);
                i--;
            }
        }
        return i > 0;
    }

    /**
     * Saves match on db
     *
     * @param u1
     * @param u2
     * @param range
     * @param lat
     * @param lon
     * @return
     */
    public long saveMatch(long u1, long u2, long range, double lat, double lon) {
        MatchEntity m = new MatchEntity();
        m.setUserId1(u1);
        m.setUserId2(u2);
        m.setParam(range);
        m.setLatitude(lat);
        m.setLongitude(lon);
        m.setDatetime(dateService.nowToString());
        m.setOutdated(false);
        m.setState("active");
        matchRepository.saveAndFlush(m);
        return m.getId();
    }

    public List<MatchEntity> getUsers10LastMatches(long user) {
        return matchRepository.findTop10ByUserId1OrUserId2OrderByIdDesc(user, user);
    }

    public LocationMatch getDtoFromEntityAndUserId(MatchEntity entity, long userId) throws CustomNotFoundException {
        long contactId = (entity.getUserId1() == userId) ? entity.getUserId2() : entity.getUserId1();
        return new LocationMatch(userService.getDtoFromUserId(contactId), entity.getParam(), entity.getDatetime());
    }

    /**
     * Get Match By Id
     *
     * @param id
     * @return
     * @throws CustomNotFoundException
     */
    public MatchEntity getMatchById(long id) throws CustomNotFoundException {
        Optional<MatchEntity> match = matchRepository.findById(id);
        if (match.isPresent()) {
            return match.get();
        } else throw new CustomNotFoundException("No match with such id");
    }

    // TO DO old matches just to return non active matches (or outdated) and getActiveMatches() to return non outdated.

    /**
     * Returns list of active matches with user
     *
     * @param user
     * @return
     */
    public List<MatchEntity> getActiveMatches(long user) {
        List<MatchEntity> matches = matchRepository.findAllByUserIDAndState(user, "active");
        List<MatchEntity> ret = new ArrayList<>();
        for (MatchEntity match : matches) {
            if (dateService.isOlderThan(match.getDatetime(), 60)) {
                match.setOutdated(true);
                matchRepository.saveAndFlush(match);
            } else {
                ret.add(match);
            }
        }
        return ret;
    }

    public List<MatchEntity> getOutdatedMatches(long user) {
        return matchRepository.findAllByUserIDAndOutdated(user, true);
    }

    /**
     * Saves match as requested
     *
     * @param userId
     * @param entity
     * @return
     * @throws CustomInternalErrorException
     */
    public MatchEntity sendRequest(long userId, MatchEntity entity) throws CustomInternalErrorException {
        if (entity.getUserId1() != userId && entity.getUserId2() != userId) {
            throw new CustomInternalErrorException("ERROR: USER ID NOT IN MATCH ENTITY. THIS SHOULD NOT BE ABLE TO HAPPEN");
        }
        entity.setState("requested");
        matchRepository.saveAndFlush(entity);
        return entity;
    }

    /**
     * Saves match as active if previously sent
     *
     * @param userId
     * @param matchId
     * @throws CustomInternalErrorException
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public void cancelRequest(long userId, long matchId) throws CustomInternalErrorException, CustomNotFoundException, CustomConflictException {
        MatchEntity entity = getMatchById(matchId);
        if (entity.getUserId1() != userId && entity.getUserId2() != userId) {
            throw new CustomInternalErrorException("ERROR: USER ID NOT IN MATCH ENTITY. THIS SHOULD NOT BE ABLE TO HAPPEN");
        }
        if (entity.getState().equals("accepted") || entity.getState().equals("canceled")) {
            throw new CustomConflictException("This request is not on active mode. It might have already been accepted");
        }
        entity.setState("active");
        matchRepository.saveAndFlush(entity);
    }
}
