package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import restapi.crud.RequestEntity;
import restapi.crud.RequestRepository;
import restapi.utils.services.date.DateService;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomNotFoundException;

import java.util.List;
import java.util.Optional;

@Service
@AllArgsConstructor
public class RequestService {
    @NonNull
    private final RequestRepository requestRepository;
    @NonNull
    private final ContactService contactService;
    @NonNull
    private final DateService dateService;

    /**
     * Return list of requests that have userID as sender or receiver
     *
     * @param userID
     * @return
     */
    public List<RequestEntity> getRequests(long userID) {
        return requestRepository.findAllBySenderIDOrReceiverID(userID, userID);
    }

    /**
     * Return list of requests that have userID as sender or receiver and state == active
     *
     * @param userID
     * @return
     */
    public List<RequestEntity> getActiveRequests(long userID) {
        return requestRepository.findAllByUserIDAndState(userID, "active");
    }

    /**
     * Return list of requests that have userID as sender or receiver and state == accepted
     *
     * @param userID
     * @return
     */
    public List<RequestEntity> getAcceptedRequests(long userID) {
        return requestRepository.findAllByUserIDAndState(userID, "accepted");
    }

    /**
     * Return list of requests that have userID as sender or receiver and state == rejected and is not older than 30 days
     *
     * @param userID
     * @return
     */
    public List<RequestEntity> getRejectedRequests(long userID) {
        List<RequestEntity> rejectRequests = requestRepository.findAllBySenderIDAndState(userID, "rejected");
        for (RequestEntity request : rejectRequests) {
            if (dateService.isOlderThan(request.getTimestamp(), 43200)) { // 30d
                requestRepository.delete(request);
                rejectRequests.remove(request);
            }
        }
        return rejectRequests;
    }

    /**
     * Return list of requests that have sender as sender and receiver as receiver
     *
     * @param sender
     * @param receiver
     * @return
     * @throws CustomNotFoundException
     */
    private RequestEntity getRequest(long sender, long receiver) throws CustomNotFoundException {
        Optional<RequestEntity> ret = requestRepository.findBySenderIDAndReceiverID(sender, receiver);
        if (ret.isPresent()) {
            return ret.get();
        } else {
            throw new CustomNotFoundException("Request with sender " + sender + " and receiver " + receiver + " not found");
        }
    }

    /**
     * Return true if there is a request with sender == user1 and receiver == user2
     *
     * @param user1
     * @param user2
     * @return
     */
    public boolean isThereAnOngoingRequest(long user1, long user2) {
        Optional<RequestEntity> ret = requestRepository.findBySenderIDAndReceiverID(user1, user2);
        if (ret.isPresent()) {
            return true;
        } else {
            ret = requestRepository.findBySenderIDAndReceiverID(user2, user1);
            return ret.isPresent();
        }
    }

    /**
     * Add request from sender to receiver if there is no conflict
     *
     * @param sender
     * @param receiver
     * @throws CustomConflictException
     */
    public void addRequest(long sender, long receiver) throws CustomConflictException {
        if (requestRepository.existsBySenderIDAndReceiverID(sender, receiver)) {
            Optional<RequestEntity> req = requestRepository.findBySenderIDAndReceiverID(sender, receiver);
            if (req.isPresent()) {
                RequestEntity request = req.get();
                if (request.getState().equals("active")) {
                    throw new CustomConflictException("There's already a request open between these users");
                } else if (request.getState().equals("cancelled") || request.getState().equals("rejected")) {
                    requestRepository.delete(request);
                } else if (request.getState().equals("accepted")) {
                    throw new CustomConflictException("These users are already friends");
                }
            } else {
                if (contactService.areFriends(sender, receiver)) {
                    throw new CustomConflictException("These users are already friends");
                }
            }
        } else if (requestRepository.existsBySenderIDAndReceiverID(receiver, sender)) {
            throw new CustomConflictException("There's already a request open between these users, but sender is receiver");
        }
        RequestEntity request = new RequestEntity();
        request.setSenderID(sender);
        request.setReceiverID(receiver);
        request.setState("active");
        request.setTimestamp(dateService.nowToString());
        requestRepository.saveAndFlush(request);
    }

    /**
     * Accept requests if exists request from sender to receiver, and it is active
     *
     * @param sender
     * @param receiver
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public void acceptRequest(long sender, long receiver) throws CustomNotFoundException, CustomConflictException {
        RequestEntity request = getRequest(sender, receiver);
        if (request.getState().equals("active")) {
            request.setState("accepted");
            contactService.addContact(sender, receiver);
            requestRepository.saveAndFlush(request);
            //requestRepository.delete(request);
        } else {
            throw new CustomConflictException("This request is not active thus it cannot be accepted");
        }
    }

    /**
     * Reject requests if exists request from sender to receiver, and it is active
     *
     * @param sender
     * @param receiver
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public void rejectRequest(long sender, long receiver) throws CustomNotFoundException, CustomConflictException {
        RequestEntity request = getRequest(sender, receiver);
        if (request.getState().equals("active")) {
            request.setState("rejected");
            requestRepository.saveAndFlush(request);
            //requestRepository.delete(request);
        } else {
            throw new CustomConflictException("This request is not active thus it cannot be rejected");
        }
    }

    public void closeRequest(long sender, long receiver) throws CustomNotFoundException, CustomConflictException {
        RequestEntity request = getRequest(sender, receiver);
        switch (request.getState()) {
            case "active":
                throw new CustomConflictException("Cannot close an active request. Please cancel it before trying again");
            case "accepted":
            case "rejected":
            case "cancelled":
                requestRepository.delete(request);
                break;
            default:
                break;
        }
    }

    /**
     * Cancel requests if exists request from sender to receiver, and it is active
     *
     * @param sender
     * @param receiver
     * @throws CustomNotFoundException
     */
    public void cancelRequest(long sender, long receiver) throws CustomNotFoundException, CustomConflictException {
        RequestEntity request = getRequest(sender, receiver);
        if (request.getState().equals("active")) {
            requestRepository.delete(request);
        } else {
            throw new CustomConflictException("This request is not active thus it cannot be rejected");
        }
    }

}
