package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import restapi.crud.UserEntity;
import restapi.crud.UserRepository;
import restapi.json.UserDTO;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomNotFoundException;
import restapi.utils.services.phone.PhoneNumberService;

import java.util.List;
import java.util.Optional;

@Service
@AllArgsConstructor
public class UserService {

    @NonNull
    private final UserRepository userRepository;
    @NonNull
    private PasswordEncoder passwordEncoder;
    @NonNull
    private UserSessionService userSessionService;
    @NonNull
    private PhoneNumberService phoneNumberService;

    /**
     * Create new user
     *
     * @param username
     * @param password
     * @param phone
     * @param email
     * @param country
     */
    public void signup(final String username, final String password, final String phone, final String email, final String country) {
        String encryptedPassword = encrypt(password);
        UserEntity user = new UserEntity(username, encryptedPassword);
        user.setName(username);
        user.setAbout("Hi! :)");
        if (email != null) {
            user.setEmail(email);
        }
        if (phone != null) {
            user.setPhone(phone);
        }
        if (country != null) {
            user.setCountry(country);
        }
        user.setAvatar(7);
        userRepository.saveAndFlush(user);
    }

    /**
     * Retrieves user from DB with id == id
     *
     * @param id
     * @return
     * @throws CustomNotFoundException
     */
    public UserEntity getUser(long id) throws CustomNotFoundException {
        Optional<UserEntity> receiver = userRepository.findById(id);
        if (receiver.isPresent()) {
            return receiver.get();
        } else {
            throw new CustomNotFoundException("User with id " + id + " not found");
        }
    }

    /**
     * Retrieves user from DB with username == username
     *
     * @param username
     * @return
     * @throws CustomNotFoundException
     */
    public UserEntity getUser(String username) throws CustomNotFoundException {
        Optional<UserEntity> receiver = userRepository.findByUsername(username);
        if (receiver.isPresent()) {
            return receiver.get();
        } else {
            throw new CustomNotFoundException("User with username " + username + " not found");
        }
    }

    /**
     * Returns id of user with username == username
     *
     * @param username
     * @return
     * @throws CustomNotFoundException
     */
    public long getIdByUsername(String username) throws CustomNotFoundException {
        return getUser(username).getId();
    }

    /**
     * Returns username of user with id == id
     *
     * @param id
     * @return
     * @throws CustomNotFoundException
     */
    public String getUsernameById(long id) throws CustomNotFoundException {
        return getUser(id).getUsername();
    }

    /**
     * Updates user's info on the database
     *
     * @param username
     * @param email
     * @param country
     * @param name
     * @param about
     * @return
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public UserEntity editUserInfo(String username, String email, String country, String name, String about) throws CustomNotFoundException, CustomConflictException {
        UserEntity userEntity = getUser(username);
        if (email != null) {
            if (userRepository.existsByEmail(email)) {
                throw new CustomConflictException("There is already a user using this email");
            }
            userEntity.setEmail(email);
        }
        if (country != null) {
            userEntity.setCountry(country);
        }
        if (name != null) {
            userEntity.setName(name);
        }
        if (about != null) {
            userEntity.setAbout(about);
        }
        userRepository.saveAndFlush(userEntity);
        return userEntity;
    }

    /**
     * Updates user's phone number on the database if valid
     *
     * @param username
     * @param phone
     * @return
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public UserEntity editUserInfo(String username, String phone) throws CustomNotFoundException, CustomConflictException {
        UserEntity userEntity = getUser(username);
        if (phone != null) {
            if (userRepository.existsByPhone(phone)) {
                throw new CustomConflictException("There is already a user using this phone");
            }
            userEntity.setPhone(phone);
        }
        userRepository.saveAndFlush(userEntity);
        return userEntity;
    }

    /**
     * Updates user's password if old matches password on DB
     *
     * @param username
     * @param old
     * @param password
     * @return
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public UserEntity changePassword(String username, String old, String password) throws CustomNotFoundException, CustomConflictException {
        UserEntity userEntity = getUser(username);
        if (!matchesEncrypted(old, userEntity.getPassword())) {
            throw new CustomConflictException("Wrong old password");
        }
        userEntity.setPassword(encrypt(password));
        userRepository.saveAndFlush(userEntity);
        return userEntity;
    }

    /**
     * Updates user's avatar id
     *
     * @param username
     * @param avatar
     * @return
     * @throws CustomNotFoundException
     * @throws CustomConflictException
     */
    public UserEntity changeAvatar(String username, int avatar) throws CustomNotFoundException, CustomConflictException {
        UserEntity userEntity = getUser(username);
        if (avatar < 1 || avatar > 16) {
            throw new CustomConflictException("avatar index invalid ( index = " + avatar + " )");
        }
        userEntity.setAvatar(avatar);
        userRepository.saveAndFlush(userEntity);
        return userEntity;
    }

    /**
     * Translates UserEntity to UserDTO
     *
     * @param u
     * @return
     */
    public UserDTO getDtoFromEntity(@NonNull UserEntity u) {
        return new UserDTO(u.getId(), u.getUsername(), u.getName(), u.getAbout(), u.getPhone(), u.getCountry(), u.getEmail(), u.getAvatar(), userSessionService.isOnline(u));
    }

    /**
     * Translates Optional of UserEntity to UserDTO
     *
     * @param user
     * @return
     */
    private @Nullable UserDTO getDtoFromOptional(@NonNull Optional<UserEntity> user) {
        if (user.isPresent()) {
            UserEntity u = user.get();
            return getDtoFromEntity(u);
        } else {
            return null;
        }
    }

    /**
     * Get UserDTO of user with id == id
     *
     * @param id
     * @return
     */
    public UserDTO getDtoById(long id) {
        Optional<UserEntity> user = userRepository.findById(id);
        return getDtoFromOptional(user);
    }

    /**
     * Get UserDTO of user with phone == phone
     *
     * @param phone
     * @param country
     * @return
     */
    public UserDTO getUserDtoByPhone(String phone, String country) {
        country = country == null ? "ES" : country;
        try {
            String phoneNumber = phone.charAt(0) != '+' ?
                    phoneNumberService.getFormattedNumberWithCountry(phone, country) :
                    phoneNumberService.getFormattedNumber(phone);
            Optional<UserEntity> user = userRepository.findByPhone(phoneNumber);
            return getDtoFromOptional(user);
        } catch (CustomConflictException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Get UserDTO of user with username == username
     *
     * @param username
     * @return
     */
    public UserDTO getDtoFromUserId(String username) {
        Optional<UserEntity> user = userRepository.findByUsername(username);
        return getDtoFromOptional(user);
    }

    /**
     * Get UserDTO of user with id == id
     *
     * @param id
     * @return
     * @throws CustomNotFoundException
     */
    public UserDTO getDtoFromUserId(long id) throws CustomNotFoundException {
        Optional<UserEntity> user = userRepository.findById(id);
        UserDTO u = getDtoFromOptional(user);
        if (u == null) {
            throw new CustomNotFoundException("User with id " + id + " not found");
        }
        return u;
    }

    /**
     * Encodes password
     *
     * @param s
     * @return
     */
    public String encrypt(String s) {
        return passwordEncoder.encode(s);
    }

    /**
     * Returns true if literal matches encoded encrypted
     *
     * @param literal
     * @param encrypted
     * @return
     */
    public boolean matchesEncrypted(String literal, String encrypted) {
        return passwordEncoder.matches(literal, encrypted);
    }

}
