package restapi.services;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import org.springframework.stereotype.Service;
import restapi.crud.UserEntity;
import restapi.crud.UserRepository;
import restapi.utils.services.date.DateService;
import restapi.utils.security.auth.LoggedUser;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;
import java.util.Optional;

@Service
@AllArgsConstructor
public class UserSessionService {

    @NonNull
    private final UserRepository userRepository;
    @NonNull
    private DateService dateService;

    /**
     * Register login to database. Saves Token and last connected.
     *
     * @param user
     * @param token
     */
    public void reportLogin(UserEntity user, String token) {
        user.setToken(encrypt(token));
        user.setLastConnection(dateService.nowToString());
        userRepository.saveAndFlush(user);
    }

    /**
     * Updates last connection on the db
     *
     * @param user
     */
    public void reportConnection(UserEntity user) {
        user.setLastConnection(dateService.nowToString());
        userRepository.saveAndFlush(user);
    }

    /**
     * Encrypts a string. Meant for token. Not retrievable.
     *
     * @param s
     * @return
     */
    public String encrypt(String s) {
        MessageDigest digest = null;
        try {
            digest = MessageDigest.getInstance("SHA-256");
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        byte[] hash = digest.digest(s.getBytes(StandardCharsets.UTF_8));
        return Base64.getEncoder().encodeToString(hash);
    }

    /**
     * Returns true if literal encrypted matches encrypted
     *
     * @param literal
     * @param encrypted
     * @return
     */
    public boolean matchesEncrypted(String literal, String encrypted) {
        return encrypt(literal).equals(encrypted);
    }

    /**
     * Returns true if token matches the encrypted token in user
     *
     * @param user
     * @param token
     * @return
     */
    public boolean isTokenInDB(UserEntity user, String token) {
        return matchesEncrypted(token, user.getToken());
    }

    /**
     * Returns false and updates the last connection datetime
     *
     * @param user
     * @return
     */
    public boolean isSessionExpired(UserEntity user) {
        // TODO activate if necessary: this 'false &&' should be eliminated to make session outdated after 30'
        if (false && dateService.isOlderThan(user.getLastConnection(), 30)) { // log out for inactivity
            user.setLastConnection(null);
            user.setToken(null);
            userRepository.saveAndFlush(user);
            return true;
        } else {
            reportConnection(user);
            return false;
        }
    }

    /**
     * Returns true if last connection was more than 30 min ago
     *
     * @param user
     * @return
     */
    public boolean isOnline(UserEntity user) {
        return user.getToken() != null && !dateService.isOlderThan(user.getLastConnection(), 30);
    }

    /**
     * Returns a LoggedUser if the token is valid, null otherwise
     *
     * @param userEntity
     * @param token
     * @return
     */
    public LoggedUser getValidSession(UserEntity userEntity, String token) {
        if (userEntity.getToken() == null) {
            return null;
        } else {
            boolean tokenInDB = isTokenInDB(userEntity, token);
            boolean sessionExpired = isSessionExpired(userEntity);
            if (!tokenInDB || sessionExpired) {
                return null;
            } else {
                return LoggedUser.builder()
                        .username(userEntity.getUsername())
                        .password(userEntity.getPassword()).build();
            }
        }
    }

    /**
     * Logs out: empties token
     *
     * @param username
     */
    public void logout(String username) {
        Optional<UserEntity> userOp = userRepository.findByUsername(username);
        if (userOp.isPresent()) {
            UserEntity user = userOp.get();
            user.setToken(null);
            userRepository.saveAndFlush(user);
        }
    }
}
