package restapi.utils.services.phone;

import com.google.i18n.phonenumbers.NumberParseException;
import com.google.i18n.phonenumbers.PhoneNumberUtil;
import com.google.i18n.phonenumbers.Phonenumber.PhoneNumber;
import org.springframework.stereotype.Service;
import restapi.utils.error.CustomConflictException;
import restapi.utils.error.CustomInternalErrorException;

@Service
public class PhoneNumberService {

    private final PhoneNumberUtil phoneNumberUtil = PhoneNumberUtil.getInstance();

    public boolean isPossibleNumber(String number, String country) {
        return phoneNumberUtil.isPossibleNumber(number, country);
    }

    public boolean isPossibleNumberForTypeFixedLine(int number, int country) {
        PhoneNumber phone = new PhoneNumber();
        phone.setCountryCode(country).setNationalNumber(number);
        return phoneNumberUtil.isPossibleNumberForType(phone, PhoneNumberUtil.PhoneNumberType.FIXED_LINE);
    }

    public boolean isPossibleNumberForTypeMobile(int number, int country) {
        PhoneNumber phone = new PhoneNumber();
        phone.setCountryCode(country).setNationalNumber(number);
        return phoneNumberUtil.isPossibleNumberForType(phone, PhoneNumberUtil.PhoneNumberType.MOBILE);
    }

    public PhoneNumber parsePhoneString(String phone) throws CustomConflictException {
        try {
            return phoneNumberUtil.parse(phone, PhoneNumber.CountryCodeSource.UNSPECIFIED.name());
        } catch (NumberParseException e) {
            e.printStackTrace();
            throw new CustomConflictException("Phone number ( " + phone + " ) is not valid");
        }
    }

    public PhoneNumber parsePhoneStringWithCountry(String phone, String country) throws CustomConflictException {
        try {
            return phoneNumberUtil.parse(phone, country);
        } catch (NumberParseException e) {
            e.printStackTrace();
            throw new CustomConflictException("Phone number ( " + phone + " ) with country ( " + country + " )is not valid");
        }
    }

    public boolean isValidNumber(PhoneNumber phone) {
        return phoneNumberUtil.isValidNumber(phone);
    }

    public boolean isValidNumberForRegion(PhoneNumber phone, String country) {
        return phoneNumberUtil.isValidNumberForRegion(phone, country);
    }

    public String formatAsInternational(PhoneNumber phone) {
        return phoneNumberUtil.format(phone, PhoneNumberUtil.PhoneNumberFormat.INTERNATIONAL);
    }

    public String formatAsNational(PhoneNumber phone) {
        return phoneNumberUtil.format(phone, PhoneNumberUtil.PhoneNumberFormat.NATIONAL);
    }

    public String formatWithoutSpaces(PhoneNumber phone) {
        return phoneNumberUtil.format(phone, PhoneNumberUtil.PhoneNumberFormat.E164);
    }

    public String getFormattedNumber(String number) throws CustomConflictException {
        PhoneNumber phone = parsePhoneString(number);
        return formatWithoutSpaces(phone);
    }

    public String getFormattedNumberWithCountry(String number, String country) throws CustomConflictException {
        PhoneNumber phone = parsePhoneStringWithCountry(number, country);
        return formatWithoutSpaces(phone);
    }

    public boolean equalPhones(String phone1, String phone2, String country) {
        try {
            country = country == null ? "ES" : country;
            String phoneNumber1 = phone1.charAt(0) != '+' ?
                    getFormattedNumberWithCountry(phone1, country) :
                    getFormattedNumber(phone1);
            String phoneNumber2 = phone2.charAt(0) != '+' ?
                    getFormattedNumberWithCountry(phone2, country) :
                    getFormattedNumber(phone2);
            return phoneNumber1.equals(phoneNumber2);
        } catch (CustomConflictException e) {
            e.printStackTrace();
            return false;
        }
    }

    public String getE164FormattedMobileNumber(String mobile, String locale) throws CustomInternalErrorException {
        try {
            PhoneNumber phoneProto = phoneNumberUtil.parse(mobile, locale);
            if (phoneNumberUtil.isValidNumber(phoneProto) && phoneNumberUtil.isPossibleNumberForType(phoneProto, PhoneNumberUtil.PhoneNumberType.MOBILE)) {
                return phoneNumberUtil.format(phoneProto, PhoneNumberUtil.PhoneNumberFormat.E164);
            }
            throw new CustomInternalErrorException("Mobile number is invalid with the provided locale");
        } catch (NumberParseException e) {
            throw new CustomInternalErrorException("Error in parsing mobile number", e);
        }
    }

}
