package restapi;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.*;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.test.web.servlet.MockMvc;
import restapi.json.ContactDTO;
import restapi.json.RestDictionary;
import restapi.json.RestList;
import restapi.json.Token;
import restapi.services.FirebaseService;

import java.util.ArrayList;
import java.util.HashMap;

import static org.hamcrest.Matchers.containsString;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
@TestMethodOrder(MethodOrderer.Alphanumeric.class)
public class ContactEndpointTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @MockBean
    private FirebaseService firebaseService;

    private static final String username1 = "userContact1";
    private static final String username2 = "userContact2";
    private static final String username3 = "userContact3";
    private static final String username4 = "userContact4";
    private static final String password = "pas";

    private static String token1;
    private static String token2;
    private static String token3;

    private static long id1;
    private static long id2;
    private static long id3;

    @AllArgsConstructor
    @NoArgsConstructor
    public static class Contact {
        @Getter
        @Setter
        ArrayList<ContactDTO> contacts;
    }

    @Test
    public void AaSetUp() throws Exception {
        Mockito.when(firebaseService.getValidPhoneFromFireAuth("+34222222221")).thenReturn("+34222222221");
        Mockito.when(firebaseService.getValidPhoneFromFireAuth("+34222222222")).thenReturn("+34222222222");
        Mockito.when(firebaseService.getValidPhoneFromFireAuth("+34222222223")).thenReturn("+34222222223");
        Mockito.when(firebaseService.getValidPhoneFromFireAuth("+34222222224")).thenReturn("+34222222224");
        // Create users
        String body = mockMvc.perform(post("/public/users/signup")
                .param("username", username1)
                .param("password", password)
                .param("authUID", "+34222222221")
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token1 = objectMapper.readValue(body, Token.class).getToken();
        body = mockMvc.perform(post("/public/users/signup")
                .param("username", username2)
                .param("password", password)
                .param("authUID", "+34222222222")
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token2 = objectMapper.readValue(body, Token.class).getToken();
        body = mockMvc.perform(post("/public/users/signup")
                .param("username", username3)
                .param("password", password)
                .param("authUID", "+34222222223")
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token3 = objectMapper.readValue(body, Token.class).getToken();
        mockMvc.perform(post("/public/users/signup")
                .param("username", username4)
                .param("password", password)
                .param("authUID", "+34222222224")
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        // Make them friends
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        this.mockMvc.perform(get("/users/current").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("phone\":\"+34222222221")));
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", username2))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", username3))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));
        headers.set("Authorization", token2);
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", username3))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));
        this.mockMvc.perform(put("/requests/accept").headers(headers)
                        .param("username", username1))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));
        headers.set("Authorization", token3);
        this.mockMvc.perform(put("/requests/accept").headers(headers)
                        .param("username", username1))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));
        this.mockMvc.perform(put("/requests/accept").headers(headers)
                        .param("username", username2))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));
        // Add location
    }

    @Test
    public void aBlockAContactOnlyShowingForBlocker() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        String response = "{\"received\":[],\"sent\":[]}";
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
        response = "online\":true},\"param\":500,\"blocked\":false},{\"user\":{";
        this.mockMvc.perform(get("/contact/all").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
        headers.set("Authorization", token2);
        String body = this.mockMvc.perform(get("/contact/all").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)))
                .andReturn().getResponse().getContentAsString();
        id1 = objectMapper.readValue(body, Contact.class).getContacts().get(0).getUser().getId();
        String response2 = "online\":true},\"param\":500,\"blocked\":true},{\"user\":{";
        this.mockMvc.perform(put("/contact/block").headers(headers)
                        .param("contactId", id1 + "")
                        .param("blocked", "true"))
                .andDo(print()).andExpect(status().isOk());
        this.mockMvc.perform(get("/contact/all").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response2)));
        headers.set("Authorization", token1);
        this.mockMvc.perform(get("/contact/all").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
    }


    @Test
    public void aExistPhones() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        ArrayList<String> phones = new ArrayList();
        phones.add("+34676656686");
        phones.add("+34222222221");
        phones.add("+34222222222");
        phones.add("+34222222224");
        phones.add("+34222222223");

        this.mockMvc.perform(get("/contacts/exist").headers(headers)
                        .param("phones", phones.toString()))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(username4)));
        this.mockMvc.perform(get("/contacts/exist?phones=+34676656686&phones=+34222222221&phones=+34222222223&phones=+34222222224").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(username4)));
        this.mockMvc.perform(get("/contacts/exist?phones=676656686&phones= 34222222224").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(username4)));
        this.mockMvc.perform(get("/contacts/exist?phones=676656686&phones= 34222222221").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("[]")));

    }


}