package restapi;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.test.web.servlet.MockMvc;
import restapi.json.Token;
import restapi.services.FirebaseService;

import static org.hamcrest.Matchers.containsString;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@SpringBootTest
@AutoConfigureMockMvc
@TestMethodOrder(MethodOrderer.Alphanumeric.class)
public class RequestEndpointTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @MockBean
    private FirebaseService firebaseService;

    private static String token1;
    private static String token2;
    private static String token3;

    @Test
    public void AaSetUp() throws Exception {
        String username = "userRequests1";
        String password = "pas";
        String body = mockMvc.perform(post("/public/users/signup")
                .param("username", username)
                .param("password", password)
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token1 = objectMapper.readValue(body, Token.class).getToken();

        username = "userRequests2";
        password = "pas";
        body = mockMvc.perform(post("/public/users/signup")
                .param("username", username)
                .param("password", password)
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token2 = objectMapper.readValue(body, Token.class).getToken();
    }

    @Test
    public void aGetRequestsIsEmptyAtFirst() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"received\":[],\"sent\":[]}")));
    }

    @Test
    public void bNewRequestAddedForBothUsers() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", "userRequests2"))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("sent\":[{\"state\":\"active\",\"user\":{\"id\":2")));

        headers.set("Authorization", token2);
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("received\":[{\"state\":\"active\",\"user\":{\"id\":1")));
    }

    @Test
    public void cNewRequestAddedForBothUsersAlreadyExists() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", "userRequests2"))
                .andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("There's already a request open between these users")));

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("sent\":[{\"state\":\"active\",\"user\":{\"id\":2")));

        headers.set("Authorization", token2);
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", "userRequests1"))
                .andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("There's already a request open between these users")));

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("received\":[{\"state\":\"active\",\"user\":{\"id\":1")));
    }


    @Test
    public void eNewRequestAlreadyFriends() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", "userRequests2"))
                .andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("These users are already friends")));
    }


    @Test
    public void bNewRequestWrongUser() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", "userRequests3"))
                .andDo(print()).andExpect(status().isNotFound())
                .andExpect(content().string(containsString("not found")));
    }

    @Test
    public void dAcceptRequestAddedForBothUsers() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token2);
        this.mockMvc.perform(put("/requests/accept").headers(headers)
                        .param("username", "userRequests1"))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));

        String response = "{\"received\":[],\"sent\":[]}";

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));

        this.mockMvc.perform(get("/contacts/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("userRequests1")));

        headers.set("Authorization", token1);
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));

        this.mockMvc.perform(get("/contacts/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("userRequests2")));
    }

    @Test
    public void dAcceptRequestNotActive() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token2);
        this.mockMvc.perform(put("/requests/accept").headers(headers)
                        .param("username", "userRequests1"))
                .andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("This request is not active thus it cannot be accepted")));

        String response = "{\"received\":[],\"sent\":[]}";

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));

        headers.set("Authorization", token1);
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
    }

    @Test
    public void eRejectRequest() throws Exception {
        String username = "userRequests3";
        String password = "pas";
        String body = mockMvc.perform(post("/public/users/signup")
                .param("username", username)
                .param("password", password)
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token3 = objectMapper.readValue(body, Token.class).getToken();

        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token2);
        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", "userRequests3"))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));

        headers.set("Authorization", token3);
        this.mockMvc.perform(put("/requests/reject").headers(headers)
                        .param("username", "userRequests2"))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request")));

        String resRequests = "{\"received\":[],\"sent\":[]}";
        String resContacts = "{\"contacts\":[]}";

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(resRequests)));

        this.mockMvc.perform(get("/contacts/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(resContacts)));

        headers.set("Authorization", token2);
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("sent\":[{\"state\":\"rejected")));
    }

    @Test
    public void dRejectRequestNotActive() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token2);
        this.mockMvc.perform(put("/requests/reject").headers(headers)
                        .param("username", "userRequests1"))
                .andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("This request is not active thus it cannot be rejected")));

        String response = "{\"received\":[],\"sent\":[]}";

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));

        headers.set("Authorization", token1);
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
    }


    @Test
    public void fAcceptRejectCancelRequestNonExistent() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        this.mockMvc.perform(put("/requests/accept").headers(headers)
                        .param("username", "userRequests3"))
                .andDo(print()).andExpect(status().isNotFound())
                .andExpect(content().string(containsString("Request with sender ")));
        this.mockMvc.perform(put("/requests/reject").headers(headers)
                        .param("username", "userRequests3"))
                .andDo(print()).andExpect(status().isNotFound())
                .andExpect(content().string(containsString("Request with sender ")));
        this.mockMvc.perform(delete("/requests/cancel").headers(headers)
                        .param("username", "userRequests3"))
                .andDo(print()).andExpect(status().isNotFound())
                .andExpect(content().string(containsString("Request with sender ")));

        String response = "{\"received\":[],\"sent\":[]}";

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));

        headers.set("Authorization", token3);
        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
    }


    @Test
    public void fCancelRequest() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);

        this.mockMvc.perform(post("/requests/new").headers(headers)
                        .param("username", "userRequests3"))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request ")));

        this.mockMvc.perform(delete("/requests/cancel").headers(headers)
                        .param("username", "userRequests3"))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Request ")));

        String response = "{\"received\":[],\"sent\":[]}";

        this.mockMvc.perform(get("/requests/mine").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
    }

}