package restapi;


import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.test.web.servlet.MockMvc;
import restapi.json.Token;
import restapi.services.FirebaseService;

import java.util.concurrent.TimeUnit;

import static org.hamcrest.Matchers.containsString;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.log;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@SpringBootTest
@AutoConfigureMockMvc
@TestMethodOrder(MethodOrderer.Alphanumeric.class)
public class UserPublicEndpointTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @MockBean
    private FirebaseService firebaseService;

    @Test
    public void aLoginUsernameNotInDB() throws Exception {
        String username = "user1";
        String password = "pas";
        this.mockMvc.perform(post("/public/users/login")
                        .param("username", username)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isNotFound())
                .andExpect(content().string(containsString("Invalid username")));
    }

    @Test
    public void bCorrectSignup() throws Exception {
        String username = "user1";
        String password = "pas";
        this.mockMvc.perform(post("/public/users/signup")
                        .param("username", username)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isCreated())
                .andExpect(content().string(containsString("token")));
    }

    @Test
    public void cCorrectLogin() throws Exception {
        String username = "user1";
        String password = "pas";
        this.mockMvc.perform(post("/public/users/login")
                        .param("username", username)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("token")));
    }

    @Test
    public void cLoginWrongPassword() throws Exception {
        String username = "user1";
        String password = "pass";
        this.mockMvc.perform(post("/public/users/login")
                        .param("username", username)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isUnauthorized())
                .andExpect(content().string(containsString("Invalid password")));
    }

    @Test
    public void aCorrectLoginAfterSignup() throws Exception {
        String username = "user2";
        String password = "pas";
        this.mockMvc.perform(post("/public/users/signup")
                .param("username", username)
                .param("password", password));
        this.mockMvc.perform(post("/public/users/login")
                        .param("username", username)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("token")));
    }

    @Test
    public void aSignupPhoneEmail() throws Exception {
        String username = "user3";
        String password = "pas";
        String phone = "+34333333333";
        String email = "user3@jud.com";
        this.mockMvc.perform(post("/public/users/signup")
                        .param("username", username)
                        .param("phone", phone)
                        .param("email", email)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isCreated())
                .andExpect(content().string(containsString("token")));
    }


    @Test
    public void bSignupUsernameAlreadyInUse() throws Exception {
        String username = "user3";
        String password = "pas";
        String phone = "+3433333333";
        String email = "user3@jud.co";
        this.mockMvc.perform(post("/public/users/signup")
                        .param("username", username)
                        .param("phone", phone)
                        .param("email", email)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("Username")));
    }

    /* FIREBASE
    @Test
    public void bSignupPhoneAlreadyInUse() throws Exception {
        String username = "user";
        String password = "pas";
        String phone = "+34333333333";
        String email = "user3@jud.co";
        this.mockMvc.perform(post("/public/users/signup")
                        .param("username", username)
                        .param("phone", phone)
                        .param("email", email)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("Phone")));
    }*/

    @Test
    public void bSignupEmailAlreadyInUse() throws Exception {
        String username = "user";
        String password = "pas";
        String phone = "+3433333333";
        String email = "user3@jud.com";
        this.mockMvc.perform(post("/public/users/signup")
                        .param("username", username)
                        .param("phone", phone)
                        .param("email", email)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isConflict())
                .andExpect(content().string(containsString("Email")));
    }

    /* FIREBASE
    @Test
    public void bLoginPhone() throws Exception {
        String password = "pas";
        String phone = "+34333333333";
        this.mockMvc.perform(post("/public/users/loginphone")
                        .param("phone", phone)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("token")));
    }*/

    @Test
    public void bLoginEmail() throws Exception {
        String password = "pas";
        String email = "user3@jud.com";
        this.mockMvc.perform(post("/public/users/loginemail")
                        .param("email", email)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("token")));
    }

    @Test
    public void bLoginPhoneWrong() throws Exception {
        String password = "pas";
        String phone = "+3433333333";
        this.mockMvc.perform(post("/public/users/loginphone")
                        .param("phone", phone)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isNotFound())
                .andExpect(content().string(containsString("phone")));
    }

    @Test
    public void bLoginEmailWrong() throws Exception {
        String password = "pas";
        String email = "user3@jud.co";
        this.mockMvc.perform(post("/public/users/loginemail")
                        .param("email", email)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isNotFound())
                .andExpect(content().string(containsString("email")));
    }

    @Test
    public void dLoginLogoutEverywhere() throws Exception {
        String username = "user1";
        String password = "pas";
        String body = mockMvc.perform(post("/public/users/login")
                .param("username", username)
                .param("password", password)
        ).andDo(print()).andExpect(status().isOk()).andReturn().getResponse().getContentAsString();
        String token1 = objectMapper.readValue(body, Token.class).getToken();

        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);

        this.mockMvc.perform(get("/users/current").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("user1")));

        TimeUnit.SECONDS.sleep(1);
        body = mockMvc.perform(post("/public/users/login")
                .param("username", username)
                .param("password", password)
        ).andDo(print()).andExpect(status().isOk()).andReturn().getResponse().getContentAsString();
        String token2 = objectMapper.readValue(body, Token.class).getToken();

        HttpHeaders headers2 = new HttpHeaders();
        headers2.add("Authorization", token2);
        this.mockMvc.perform(get("/users/current").headers(headers2))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("user1")));

        String token3 = "eyJhbGciOiJIUzI1NiIsInppcCI6IkdaSVAifQ.H4sIAAAAAAAA_6tWyiwuVrJSyktNLUqqVNJRykwsUbIyNDM2tTS2MLM00VFKrSgAC5gZGJkagARKi1OL8hJzU4G6QExDpVoAeBv-kkUAAAA.q9duilrQbgoMRo-0GZ_Spr3am4q6VVyXdq22M0Ft7ew";

        boolean equals = token1.equals(token2);

        HttpHeaders headers3 = new HttpHeaders();
        headers3.add("Authorization", token3);
        this.mockMvc.perform(get("/users/current").headers(headers3))
                .andDo(print()).andExpect(status().isUnauthorized());
        this.mockMvc.perform(get("/users/current").headers(headers))
                .andDo(print()).andExpect(status().isUnauthorized());

    }
}