import { Component, OnInit } from '@angular/core';
import { AlertController } from '@ionic/angular';
import { pickerController } from '@ionic/core';
import { parsePhoneNumber } from 'libphonenumber-js';
import { ContactPrefData } from 'src/app/interfaces/data-classes';
import { CommonUtilsService } from 'src/app/providers/common-utils.service';
import { ApiService } from 'src/app/providers/server/api.service';
import { StorageService } from 'src/app/providers/storage.service';

@Component({
  selector: 'app-contact-detail',
  templateUrl: './contact-detail.page.html',
  styleUrls: ['./contact-detail.page.scss'],
})
export class ContactDetailPage implements OnInit {

  contactId: number;
  name = '';
  username = '';
  about = '';
  phone = '';
  email = '';
  country = '';
  online: boolean;
  avatar: number;

  range = '';
  blocked: boolean;

  pickerOptions = [100, 200, 300, 400, 500, 750, 1000, 2000, 3000, 4000, 5000, 7500, 10000, 20000, 50000];

  configurationOptions = [
    { title: 'Change notification range', click: 1, iconPath: '/assets/icons/user.svg' },
    { title: 'Open chat', click: 2, iconPath: '/assets/icons/whatsapp.svg'}
    //{ title: 'Change phone number', link: 'change-phone' }
  ];

  constructor(
    private storage: StorageService,
    private api: ApiService,
    private alertCtrl: AlertController,
    private utils: CommonUtilsService
  ) { }

  ngOnInit(): void {
    
  }

  ionViewWillEnter() {
    this.storage.get('contact').then(contact => {
      if (contact.user === null) {
        this.loadContactDetails(contact);
      } else {
        this.processContactInfo(contact);
      }
    })
  }

  loadContactDetails(contactId) {
    this.storage.get('authToken').then(token => {
      this.api.getContactDetails(token, contactId).subscribe({
        next: response => {
          this.processContactInfo(response);
        }
      })
    })
  }

  processContactInfo(contact) {
    this.range = contact.param;
    this.blocked = contact.blocked;
    this.contactId = contact.user.id;
    this.username = contact.user.username;
    this.name = contact.user.name;
    this.about = contact.user.about;
    try { if (contact.user.phone != null) this.phone = parsePhoneNumber(contact.user.phone).formatInternational(); }
    catch (e) { this.phone = contact.user.phone; }
    this.email = contact.user.email;
    this.country = contact.user.country;
    this.online = contact.user.online;
    this.avatar = contact.user.avatar;
  }

  getAvatar() {
    return this.utils.getUserAvatarLink(this.avatar);
  }

  async onClick(op: number) {
    switch (op) {
      case 1:
        var options = [];
        for (let r in this.pickerOptions) {
          options.push({ text: this.pickerOptions[r] + ' m', value: this.pickerOptions[r] });
        }

        const picker = await pickerController.create({
          columns: [
            {
              name: 'range',
              options: options
            }
          ],
          buttons: [
            {
              text: 'Cancel',
              role: 'cancel',
            },
            {
              text: 'Confirm',
              handler: (value) => {
                console.log(`Got Value ${value.range.text}`);
                this.sendNewParamRange(value.range.value);
              },
            },
          ],
        });

        await picker.present();

        break;
      case 2:
        this.utils.openChat(this.phone);
        break;
    }
  }

  async presentConfirmBlock() {
    const alert = await this.alertCtrl.create({
      header: 'Confirm Block',
      message: 'Are you sure you want to block this user?',
      buttons: [
        {
          text: 'Cancel',
          role: 'cancel',
          cssClass: 'secondary',
          handler: () => {
            console.log('Cancel clicked');
          }
        },
        {
          text: 'Block',
          role: 'accept',
          cssClass: 'danger',
          handler: () => {
            this.storage.get('authToken').then(token => {
              var data = new ContactPrefData(this.contactId);
              data.blocked = true;
              this.api.updateBlocked(token, data).subscribe({
                next: response => {
                  console.log(response);
                  this.blocked = true;
                }
              })
            });
          }
        }
      ]
    });
    await alert.present();
  }

  async presentConfirmUnblock() {
    const alert = await this.alertCtrl.create({
      header: 'Confirm Block',
      message: 'Are you sure you want to unblock this user?',
      buttons: [
        {
          text: 'Cancel',
          role: 'cancel',
          cssClass: 'secondary',
          handler: () => {
            console.log('Cancel clicked');
          }
        },
        {
          text: 'Unblock',
          role: 'accept',
          cssClass: 'primary',
          handler: () => {
            this.storage.get('authToken').then(token => {
              var data = new ContactPrefData(this.contactId);
              data.blocked = false;
              this.api.updateBlocked(token, data).subscribe({
                next: response => {
                  console.log(response);
                  this.blocked = false;
                }
              })
            });
          }
        }
      ]
    });
    await alert.present();
  }

  sendNewParamRange(value: number) {
    this.storage.get('authToken').then(token => {
      var data = new ContactPrefData(this.contactId);
      data.range = value;
      this.api.updateRange(token, data).subscribe({
        next: response => {
          console.log(response);
          this.range = value + '';
        }
      })
    });
  }

}
