import { Component, OnDestroy, OnInit } from '@angular/core';
import { LocationData, MatchData, UserData } from 'src/app/interfaces/data-classes';
import { LocalNotificationService } from 'src/app/providers/os/local-notification.service';
//import { FirebaseCloudMessagingService } from 'src/app/providers/firebase-cloud-messaging.service';
import { BackgroundService } from 'src/app/providers/os/background.service';
import { StorageService } from 'src/app/providers/storage.service';
import { WebSocketService } from 'src/app/providers/server/web-socket.service';
import { Router } from '@angular/router';
import { ActionSheetController } from '@ionic/angular';
import { ApiService } from 'src/app/providers/server/api.service';
import { CommonUtilsService } from 'src/app/providers/common-utils.service';

@Component({
  selector: 'app-location',
  templateUrl: './location.page.html',
  styleUrls: ['./location.page.scss'],
})
export class LocationPage implements OnInit, OnDestroy {

  name: string;

  firstTime = true;

  sentMatchRequests: MatchData[];
  receivedMatchRequests: MatchData[];

  notificationMatches: MatchData[];

  constructor(
    private webSocketAPI: WebSocketService,
    private storage: StorageService,
    private backgroundService: BackgroundService,
    private api: ApiService,
    private localNotificationService: LocalNotificationService,
    private router: Router,
    public actionSheetController: ActionSheetController,
    private utils: CommonUtilsService
    //private fcmService: FirebaseCloudMessagingService
  ) { }

  ngOnInit() {
    this.localNotificationService.requestPermisions();
    this.connect();
  }

  ngOnDestroy(): void {
    this.disconnect();
  }

  ionViewWillEnter() {
    this.loadMatches();
    /*if(!this.webSocketAPI.checkConnection() && !this.firstTime) {
      this.connect();
    }//*/
  }

  loadMatches() {
    this.storage.get('authToken').then(token => {
      this.api.getActiveMatches(token).subscribe({
        next: response => {
          this.sentMatchRequests = response.sent;
          this.receivedMatchRequests = response.received;
          this.notificationMatches = response.active;
        }
      });
    });
  }

  connect() {
    this.storage.get('username').then(id => {
      this.storage.get('authToken').then(token => {
        this.webSocketAPI.initConnection(id, token);
        this.webSocketAPI.locationPage = this;
        this.backgroundService.initBackground();
        if (this.firstTime) this.firstTime = false;
      })
    });
  }

  disconnect() {
    this.webSocketAPI.disconnect();
    this.backgroundService.stopBackgroundFetch();
  }

  sendMessage() {
    this.backgroundService.getCurrentPosition().then(coords => {
      this.storage.get('username').then(username => {
        this.storage.get('authToken').then(token => {
          this.api.updateLocation(token, new LocationData(coords.latitude.toString(), coords.longitude.toString(), username)).subscribe({
            next: response => {
              console.log(response);
            }
          });
        })
      });
    });
  }

  addMatch(match: MatchData) {
    if (!this.notificationMatches) { this.notificationMatches = [match] } else {
      this.notificationMatches = [match].concat(this.notificationMatches);
    }
  }

  async presentMatchActionSheet(match) {
    const actionSheet = await this.actionSheetController.create({
      header: 'New Match',
      cssClass: 'my-custom-class',
      buttons: [
        {
          text: 'Contact',
          handler: () => this.sendMatch(match),
        },
        {
          text: 'Reject',
          handler: () => this.rejectMatch(match),
        },
        {
          text: 'Edit Configuration',
          handler: () => this.editConfig(match),
        },
        {
          text: 'Cancel',
          role: 'cancel',
          handler: () => { },
        },
      ],
    });
    await actionSheet.present();
  }

  sendMatch(match) {
    this.storage.get('authToken').then(token => {
      this.api.newMatchRequest(token, match.id).subscribe({
        next: response => {
          this.loadMatches();
        }
      })
    })
  }

  rejectMatch(match) {
    //this.api.matchReject(match);
    this.storage.get('authToken').then(token => {
      this.api.rejectMatchRequest(token, match.id).subscribe({
        next: response => {
          this.loadMatches();
        }
      })
    })
  }

  async presentConnectActionSheet(match) {
    const actionSheet = await this.actionSheetController.create({
      header: 'Meet',
      cssClass: 'my-custom-class',
      buttons: [
        {
          text: 'Open Chat',
          handler: () => {
            this.utils.openChat(match);
            this.acceptMatch(match);
          },
        },
        {
          text: 'Interested',
          handler: () => this.acceptMatch(match),
        },
        {
          text: 'Edit Configuration',
          handler: () => this.editConfig(match),
        },
        {
          text: 'Cancel',
          role: 'cancel',
          handler: () => { },
        },
      ],
    });
    await actionSheet.present();
  }

  acceptMatch(match) {
    this.storage.get('authToken').then(token => {
      this.api.acceptMatchRequest(token, match.id).subscribe({
        next: response => {
          this.loadMatches();
        }
      })
    })
  }

  editConfig(match) {
    this.storage.set('contact', match.user.id).then(() =>
      this.router.navigate(['tabs/contacts/detail'])
    )
  }

  async presentCancelActionSheet(match) {
    const actionSheet = await this.actionSheetController.create({
      header: 'Meet',
      cssClass: 'my-custom-class',
      buttons: [
        {
          text: 'Cancel request',
          handler: () => this.cancelMatch(match),
        },
        {
          text: 'Edit Configuration',
          handler: () => this.editConfig(match),
        },
        {
          text: 'Cancel',
          role: 'cancel',
          handler: () => { },
        },
      ],
    });
    await actionSheet.present();
  }

  cancelMatch(match) {
    this.storage.get('authToken').then(token => {
      this.api.cancelMatchRequest(token, match.id).subscribe({
        next: response => {
          this.loadMatches();
        }
      })
    })
  }

  seeOlder() {
    this.router.navigate(['tabs/location/old'], { replaceUrl: false });
  }

}
