import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AlertController, IonRouterOutlet, ModalController } from '@ionic/angular';
import { formatNumber, parsePhoneNumber } from 'libphonenumber-js';
import { UserData } from 'src/app/interfaces/data-classes';

import { ApiService } from 'src/app/providers/server/api.service';
import { LogoutService } from 'src/app/providers/auth/logout.service';
import { StorageService } from 'src/app/providers/storage.service';
import { CommonUtilsService } from 'src/app/providers/common-utils.service';
import { UserDetailEditModal } from '../user-detail-edit-modal/user-detail-edit-modal.page';

@Component({
  selector: 'app-user-detail',
  templateUrl: './user-detail.page.html',
  styleUrls: ['./user-detail.page.scss'],
})
export class UserDetailPage implements OnInit {

  name = '';
  username = '@';
  about = '';
  phone = '';
  email = '';
  country = '';
  avatar = '';

  newName = '';
  newAbout = '';
  editing = false;

  configurationOptions = [
    { title: 'Change password', link: 'change-password', iconPath: '../../assets/icons/key.svg' },
    //{ title: 'Change phone number', link: 'change-phone' }
  ];

  constructor(
    public storage: StorageService,
    public api: ApiService,
    public logoutService: LogoutService,
    private router: Router,
    private alertCtrl: AlertController,
    private utils: CommonUtilsService,
    private modalController: ModalController
  ) { }

  ngOnInit() {

  }

  ionViewWillEnter() {
    this.loadUserInfo();
  }

  doRefresh(event) {
    this.loadUserInfo();
    setTimeout(() => {
      event.target.complete();
    }, 500);
  }

  parseResponse(response) {
    this.username = '@' + response.username;
    this.name = this.addTags(response.name);
    this.newName = this.addTags(response.name);
    this.about = this.addTags(response.about);
    this.newAbout = this.addTags(response.about);
    try { if (response.phone != null) this.phone = parsePhoneNumber(response.phone).formatInternational(); }
    catch (e) { this.phone = response.phone; }
    this.email = response.email;
    this.country = response.country;
    this.getAvatar(response.avatar);
    this.storage.set('userID', response.id);
  }

  loadUserInfo() {
    this.storage.get('authToken').then(token => {
      this.api.getUserInfo(token).subscribe({
        next: response => {
          this.parseResponse(response);
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }

  async presentConfirm() {
    const alert = await this.alertCtrl.create({
      header: 'Confirm logout',
      message: 'Are you sure you want to logout?',
      buttons: [
        {
          text: 'Cancel',
          role: 'cancel',
          cssClass: 'secondary',
          handler: () => {
            console.log('Cancel clicked');
          }
        },
        {
          text: 'Logout',
          role: 'accept',
          cssClass: 'alertButton',
          handler: () => {
            this.storage.get('authToken').then(token => {
              this.api.logout(token).subscribe({
                next: response => {
                  console.log(response);
                  this.logoutService.logout();
                }
              })
            });
          }
        }
      ]
    });
    await alert.present();
  }

  editInfo() {
    this.editing = true;
  }

  async changeImage() {
    const modal = await this.modalController.create({
      component: UserDetailEditModal
    });
    modal.onWillDismiss().then(() => {
      this.loadUserInfo();
    })
    return await modal.present();
  }

  endEdit() {
    var user = new UserData();
    if (this.name != this.newName) {
      user.name = this.filterTags(this.newName);//.replace(/<div>/, '').replace(/<\/div>/, '');
    }
    if (this.about != this.newAbout) {
      user.about = this.filterTags(this.newAbout);
    }
    this.storage.get('authToken').then(token => {
      this.api.editUserInfo(token, user).subscribe({
        next: response => {
          this.editing = false;
          this.parseResponse(response);
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }

  filterTags(s: string) : string {
    return s.split('<div>').join('\n').split('</div>').join('').split('<br>').join('\n');
    //.replace(/<div>/, '').replace(/<\/div>/, '');
  }

  addTags(s: string) : string {
    return s.split('&nbsp;').join('\n');
    //.replace(/<div>/, '').replace(/<\/div>/, '');
  }

  onNameChange(data) {
    this.newName = data;
  }

  onAboutChange(data) {
    this.newAbout = data;
  }

  onClick(link: string) {
    this.router.navigate([link]);
  }

  getAvatar(id) {
    this.avatar = this.utils.getUserAvatarLink(id);
    //return require(this.utils.getUserAvatarLink(id));
  }

}
